--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert Farming Tablet [BunkerSiloOverview] AddOn
## Description: UnitConvert AddOn for handling FarmingTablet texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/05/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "FarmingTablet [BunkerSiloOverview]"
ucMod.addOns[ucAddOnIndex].source	= "FarmingTablet_App_BunkerSiloOverview"
ucMod.addOns[ucAddOnIndex].version	= "1.0.0.0"

local ucFarmingTabletBunkerSiloOverview = ucMod.addOns[ucAddOnIndex]
local FarmingTablet_App_BunkerSiloOverview = nil

function ucFarmingTabletBunkerSiloOverview:initialize()
	local target = ucFarmingTabletBunkerSiloOverview.target
	if target then
		target.draw_BunkerSiloOverview_page = Utils.overwrittenFunction(target.draw_BunkerSiloOverview_page, ucFarmingTabletBunkerSiloOverview.draw_BunkerSiloOverview_page)

		FarmingTablet_App_BunkerSiloOverview = target
		return true
	end
end

function ucFarmingTabletBunkerSiloOverview:draw_BunkerSiloOverview_page(superFunc)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "backScreen_BunkerSilo").id, 0.019, 0.0163, 1, 0.964)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "menuBunkerSilo").id, 0.43, 0.153, 0.5, 0.65)
	g_farmingTablet.button.drawButtonByCategory("BUNKERSILOOVERVIEW_PAGE")

	if FarmingTablet_App_BunkerSiloOverview.data[FarmingTablet_App_BunkerSiloOverview.activeSilo] ~= nil then
		local object = FarmingTablet_App_BunkerSiloOverview.data[FarmingTablet_App_BunkerSiloOverview.activeSilo]
		local text = string.format("Bunker Silo  %s / %s", FarmingTablet_App_BunkerSiloOverview.activeSilo, table.getn(FarmingTablet_App_BunkerSiloOverview.data))

		-- [UC] Convert FillTypes
		local ucFillType = object.inputFillType
		if object.state ~= BunkerSilo.STATE_FILL then
			ucFillType = object.outputFillType
		end

		local ucFillUnit = ucMod:getFillUnit(ucFillType, ucMod.UNIT_SET_STORAGE)

		local ucFillLevel = object.fillLevel
		ucFillLevel = ucMod:convertValue(ucFillLevel, ucFillUnit.index, ucFillType)
		ucFillLevel = numToStr(ucFillLevel, ucFillUnit.precision, "ceil", true, false)

		local ucCompactedFillLevel = object.compactedFillLevel
		ucCompactedFillLevel = ucMod:convertValue(ucCompactedFillLevel, ucFillUnit.index, ucFillType)
		ucCompactedFillLevel = numToStr(ucCompactedFillLevel, ucFillUnit.precision, "ceil", true, false)

		setTextBold(true)
		setTextColor(1, 1, 1, 1)
		local x = 0.279 - getTextWidth(0.030, text) / 2
		renderText(x, 0.686, 0.030, text)

		if object.fillLevel > 0 then
			if object.compactedPercent >= 100 then
				setTextColor(0.2122, 0.5271, 0.0307, 1)
			elseif object.compactedPercent >= 50 then
				setTextColor(0.9301, 0.2874, 0.0130, 1)
			else
				setTextColor(0.8069, 0.0097, 0.0097, 1)
			end
		else
			setTextColor(1, 1, 1, 1)
		end

		renderText(0.505, 0.487, 0.025, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "compacted") .. " :")
		renderText(0.658, 0.487, 0.025, tostring(string.format("%d", object.compactedPercent)) .. "  %")

		-- [UC] Display Converted Compacted FillLevel
		renderText(0.761, 0.487, 0.025, ucCompactedFillLevel)
		renderText(0.844, 0.487, 0.025, ucFillUnit.nameI18NShort)

		--renderText(0.761, 0.487, 0.025, tostring(string.format("%d", object.compactedFillLevel)))
		--renderText(0.844, 0.487, 0.025, "l")

		if object.state ~= 0 then
			if object.fermentingPercent >= 100 and object.fillLevel > 0 then
				setTextColor(0.2122, 0.5271, 0.0307, 1)
			elseif object.fermentingPercent >= 50 and object.fillLevel > 0 then
				setTextColor(0.9301, 0.2874, 0.0130, 1)
			else
				setTextColor(0.8069, 0.0097, 0.0097, 1)
			end
		else
			setTextColor(1, 1, 1, 1)
		end
		renderText(0.505, 0.41, 0.025, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "fermenting") .. " :")
		renderText(0.658, 0.41, 0.025, tostring(string.format("%d", object.fermentingPercent)) .. "  %")
		local fermentingHours = object.fermentingTime / 60 / 60
		local fermentingDays = object.fermentingTime / 60 / 60 / 24

		if fermentingHours >= 0 and fermentingHours < 1 or fermentingHours >= 2 and fermentingHours < 24 then
			renderText(0.769, 0.41, 0.025, tostring(string.format("%d", fermentingHours)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "hours"))
		elseif fermentingHours >= 1 and fermentingHours < 2 then
			renderText(0.769, 0.41, 0.025, tostring(string.format("%d", fermentingHours)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "hour"))
		end

		if fermentingHours >= 24 then
			if fermentingDays < 2 then
				renderText(0.769, 0.41, 0.025, tostring(string.format("%d", fermentingDays)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "day"))
			else
				renderText(0.769, 0.41, 0.025, tostring(string.format("%d", fermentingDays)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "days"))
			end
		end

		setTextColor(1, 1, 1, 1)
		renderText(0.668, 0.625, 0.019, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "estimatedTimeOne"))
		renderText(0.668, 0.597, 0.019, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "estimatedTimeTwo"))
		renderText(0.668, 0.569, 0.019, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "estimatedTimeThree"))

		if object.fillLevel > 0 then
			local estimatedHours = object.fermentingDuration / 60 / 60
			local estimatedDays = object.fermentingDuration / 60 / 60 / 24

			if estimatedHours >= 0 and estimatedHours < 1 or estimatedHours >= 2 and estimatedHours < 24 then
				renderText(0.807, 0.598, 0.020, tostring(string.format("%d", estimatedHours)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "hours"))
			elseif estimatedHours >= 1 and estimatedHours < 2 then
				renderText(0.807, 0.598, 0.020, tostring(string.format("%d", estimatedHours)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "hour"))
			end

			if estimatedHours >= 24 then
				if estimatedDays < 2 then
					renderText(0.807, 0.598, 0.020, tostring(string.format("%d", estimatedDays)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "day"))
				else
					renderText(0.807, 0.598, 0.020, tostring(string.format("%d", estimatedDays)) .. "  " .. g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "days"))
				end
			end
		end

		renderText(0.671, 0.736, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "currentFillLevel"))

		-- [UC] Display Converted FillLevel
		renderText(0.770, 0.736, 0.021, ucFillLevel)
		renderText(0.867, 0.736, 0.021, ucFillUnit.nameI18NShort)

		--renderText(0.770, 0.736, 0.021, tostring(string.format("%d", object.fillLevel)))
		--renderText(0.867, 0.736, 0.021, "l")

		setTextAlignment(RenderText.ALIGN_CENTER)
		if object.state == 0 then
			setTextColor(1, 1, 1, 1)
			renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "stateOne"))
		elseif object.state == 1 then
			if object.fermentingPercent <= 50 and object.fillLevel > 0 then
				setTextColor(0.8069, 0.0097, 0.0097, 1)
			else
				setTextColor(0.9301, 0.2874, 0.0130, 1)
			end
			renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "stateTwo"))
		elseif object.state == 2 then
			setTextColor(0.2122, 0.5271, 0.0307, 1)
			renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "stateThree"))
		elseif object.state == 3 then
			setTextColor(1, 1, 1, 1)
			if object.isOpenedAtFront and not object.isOpenedAtBack then
				renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "openFront"))
			elseif object.isOpenedAtBack and not object.isOpenedAtFront then
				renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "openBack"))
			end
			if object.isOpenedAtFront and object.isOpenedAtBack then
				renderText(0.77, 0.680, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "bunkerOpen"))
			end
		end

		setTextColor(1, 1, 1, 1)
		if object.fillLevel > 0 then
			if object.state == 0 or object.state == 1 then
				local inputImage = FillUtil.fillTypeIndexToDesc[object.inputFillType]
				local inputPicture = createImageOverlay(inputImage.hudOverlayFilename)
				renderOverlay(inputPicture, 0.50, 0.60, 0.093, 0.171)
				local inputName = FillUtil.fillTypeIntToName[object.inputFillType]
				renderText(0.550, 0.56, 0.021, inputImage.nameI18N)
			elseif object.state == 2 or object.state == 3 then
				local outputImage = FillUtil.fillTypeIndexToDesc[object.outputFillType]
				local outputPicture = createImageOverlay(outputImage.hudOverlayFilename)
				renderOverlay(outputPicture, 0.50, 0.60, 0.093, 0.171)
				renderText(0.550, 0.56, 0.021, outputImage.nameI18N)
			end
		else
			renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "bunkerEmpty").id, 0.50, 0.60, 0.096, 0.171)
			renderText(0.550, 0.56, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "bunkerEmpty"))
		end

		if object.state ~= 0 then
			if object.isOpenedAtBack then
				renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "bunkerSilage").id, 0.472, 0.205, 0.21, 0.133)
			else
				renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "bunkerCover").id, 0.472, 0.205, 0.21, 0.133)
			end
			if object.isOpenedAtFront then
				renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "bunkerSilage").id, 0.676, 0.205, 0.21, 0.133)
			else
				renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "bunkerCover").id, 0.676, 0.205, 0.21, 0.133)
			end
		else
			renderText(0.680, 0.265, 0.030, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_BunkerSiloOverview.name, "bunkerOpen"))
		end

		renderOverlay(FarmingTablet_App_BunkerSiloOverview.pda, 0.143, 0.18, 0.27, 0.48)
		local x, _, z = getWorldTranslation(object.node)
		x = 0.278 + (0.27 / FarmingTablet_App_BunkerSiloOverview.baseMapPDAwidth * x) - 0.0025
		z = 0.42 + (0.48 / FarmingTablet_App_BunkerSiloOverview.baseMapPDAwidth * -z) - 0.005
		if object.id == FarmingTablet_App_BunkerSiloOverview.activeSilo then
			renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "onBunkerSiloOverview").id, x, z, 0.005, 0.01)
		end
	else
		renderText(0.46, 0.721, 0.026, "Keine Bunker Silos gefunden / No Bunker Silos found")
	end
end
