--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert Farming Tablet [FactoryExtension] AddOn
## Description: UnitConvert AddOn for handling FarmingTablet texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/05/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "FarmingTablet [FactoryExtension]"
ucMod.addOns[ucAddOnIndex].source	= "FarmingTablet_App_FactoryExtension"
ucMod.addOns[ucAddOnIndex].version	= "1.2"

local ucFarmingTabletFactoryExtension = ucMod.addOns[ucAddOnIndex]
local FarmingTablet_App_FactoryExtension = nil

function ucFarmingTabletFactoryExtension:initialize()
	local target = ucFarmingTabletFactoryExtension.target
	if target then
		target.draw_Fabrikscript_infos = Utils.overwrittenFunction(target.draw_Fabrikscript_infos, ucFarmingTabletFactoryExtension.draw_Fabrikscript_infos)
		target.getFabrikscriptInfo = Utils.overwrittenFunction(target.getFabrikscriptInfo, ucFarmingTabletFactoryExtension.getFabrikscriptInfo)

		FarmingTablet_App_FactoryExtension = target
		return true
	end
end

function ucFarmingTabletFactoryExtension:draw_Fabrikscript_infos(superFunc)
	-- [UC] Get Base Object (For Retrieving FillTypes)
	local ucObjectId = FarmingTablet_App_FactoryExtension.object.id
	local ucBaseObject = FarmingTablet_App_FactoryExtension.objects[ucObjectId]

	-- [UC] Conversion Function
	local function ucConvertValue(object, value)
		local ucFillUnit = ucMod:getUnit(ucMod.UNIT_LITER)
		if object and object.acceptedFillTypes then
			local fillTypes = {}
			for fillType, isValid in pairs(object.acceptedFillTypes) do
				if isValid then
					table.insert(fillTypes, fillType)
				end
			end

			if getTableLength(fillTypes) > 0 then
				local bestFillType = ucMod:getHeaviestFillType(fillTypes)
				ucFillUnit = ucMod:getFillUnit(fillTypes, ucMod.UNIT_SET_STORAGE)
				value = ucMod:convertValue(value, ucFillUnit.index, bestFillType)
			end
		end
		return numToStr(value, 0, "ceil", true, false), ucFillUnit.nameI18NShort -- [UC] Precision must be zero to avoid text overlap
	end

	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "backScreenAlpha2").id, 0.11, 0.2, 0.61, 0.45)
	setTextColor(g_farmingTablet.utils.getActiveAppTextColor())
	renderText(0.2, 0.75, 0.035, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_FactoryExtension.name, "Detailoverview")..FarmingTablet_App_FactoryExtension.object.text)

	if FarmingTablet_App_FactoryExtension.object.showProduktPerHour then
		-- [UC] Convert Product Per Hour
		local ucTmpTable = {}
		for _, ucOutputObject in pairs(ucBaseObject.Produkte) do
			if ucOutputObject.acceptedFillTypes then
				for fillType, isValid in pairs(ucOutputObject.acceptedFillTypes) do
					if isValid then
						ucTmpTable[fillType] = fillType
					end
				end
			end
		end

		local ucProductPerHour = FarmingTablet_App_FactoryExtension.object.produktPerHour
		local ucFillUnit = ucMod:getUnit(ucMod.UNIT_LITER)

		local ucFillTypes = {}
		for _, fillType in pairs(ucTmpTable) do
			table.insert(ucFillTypes, fillType)
		end

		if getTableLength(ucFillTypes) > 0 then
			local ucBestFillType = ucMod:getHeaviestFillType(ucFillTypes)
			ucFillUnit = ucMod:getFillUnit(ucFillTypes, ucMod.UNIT_SET_STORAGE)
			ucProductPerHour = ucMod:convertValue(ucProductPerHour, ucFillUnit.index, ucBestFillType)
		end
		ucProductPerHour = numToStr(ucProductPerHour, ucFillUnit.precision, "ceil", true, false)
		renderText(0.28, 0.68, 0.026, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_FactoryExtension.name, "ProduktPerHour")..string.format("%s %s", ucProductPerHour, ucFillUnit.nameI18NShort))
		--renderText(0.28, 0.68, 0.026, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_FactoryExtension.name, "ProduktPerHour")..tostring(FarmingTablet_App_FactoryExtension.object.produktPerHour))
	end

	renderText(0.12, 0.6, 0.029, FarmingTablet_App_FactoryExtension.object.InputName)
	renderText(0.43, 0.6, 0.029, FarmingTablet_App_FactoryExtension.object.OutputName)

	local i = 0
	for k, v in pairs(FarmingTablet_App_FactoryExtension.object.rohstoffe) do
		renderText(0.12, 0.54 - (i * 0.04), 0.024, v.name)

		-- [UC] Convert Fill Level and Capacity
		local ucInputName = ucBaseObject.StoffeIdToName[v.id]
		local ucInputObject = ucBaseObject.Rohstoffe[ucInputName]
		local ucFillLevel, ucFillUnitName = ucConvertValue(ucInputObject, v.fillLevel)
		local ucCapacity = ucConvertValue(ucInputObject, v.capacity)
		renderText(0.22, 0.54 - (i * 0.04), 0.024, string.format("%s / %s %s", ucFillLevel, ucCapacity, ucFillUnitName))

		--renderText(0.22, 0.54 - (i * 0.04), 0.024, string.format("%.f / %s", v.fillLevel, v.capacity))
		renderText(0.35, 0.54 - (i * 0.04), 0.024, string.format("%.f %%", (v.fillLevel / v.capacity * 100)))
		i = i + 1
	end

	i = 0
	for k, v in pairs(FarmingTablet_App_FactoryExtension.object.produkte) do
		renderText(0.43, 0.54 - (i * 0.04), 0.024, v.name)

		-- [UC] Convert Fill Level and Capacity
		local ucOutputName = ucBaseObject.StoffeIdToName[v.id]
		local ucOutputObject = ucBaseObject.Produkte[ucOutputName]
		local ucFillLevel, ucFillUnitName = ucConvertValue(ucOutputObject, v.fillLevel)
		local ucCapacity = ucConvertValue(ucOutputObject, v.capacity)
		renderText(0.55, 0.54 - (i * 0.04), 0.024, string.format("%s / %s %s", ucFillLevel, ucCapacity, ucFillUnitName))

		--renderText(0.55, 0.54 - (i * 0.04), 0.024, string.format("%.f / %s", v.fillLevel, v.capacity))
		renderText(0.66, 0.54 - (i * 0.04), 0.024, string.format("%.f %%", (v.fillLevel / v.capacity*100)))
		i = i + 1
	end

	renderOverlay(FarmingTablet_App_FactoryExtension.object.pic, 0.74, 0.5, 0.16, 0.284)
	g_farmingTablet.button.drawButtonByCategory("Fabrikscript_PAGE_INFOS")
end

function ucFarmingTabletFactoryExtension:getFabrikscriptInfo(superFunc)
	local data = {}
	data.numObjects = 0
	data.objects = {}
	for _, object in pairs(FarmingTablet_App_FactoryExtension.objects) do
		data.numObjects = data.numObjects + 1
		data.objects[data.numObjects] = {}
		data.objects[data.numObjects].nodeId = object.nodeId
		data.objects[data.numObjects].id = data.numObjects
		if g_i18n:hasText(getName(object.nodeId)) then
			data.objects[data.numObjects].name = g_i18n:getText(getName(object.nodeId))
		elseif FarmingTablet_App_FactoryExtension.mapText[getName(object.nodeId)] ~= nil then
			data.objects[data.numObjects].name = FarmingTablet_App_FactoryExtension.mapText[getName(object.nodeId)]
		else
			data.objects[data.numObjects].name = getName(object.nodeId)
		end

		local RohstoffText = "Rohstoffe:"
		if g_i18n:hasText(object.InputName) then
			RohstoffText = g_i18n:getText(object.InputName)
		elseif g_i18n:hasText("Rohstoffe") then
			RohstoffText = g_i18n:getText("Rohstoffe")
		end
		local ProdukteText = "Produkte:"
		if g_i18n:hasText(object.OutputName) then
			ProdukteText = g_i18n:getText(object.OutputName)
		elseif g_i18n:hasText("Produkte") then
			ProdukteText = g_i18n:getText("Produkte")
		end
		data.objects[data.numObjects].InputName = RohstoffText
		data.objects[data.numObjects].OutputName = ProdukteText

		if g_i18n:hasText(data.objects[data.numObjects].name) then
			data.objects[data.numObjects].text = g_i18n:getText(data.objects[data.numObjects].name)
		else
			data.objects[data.numObjects].text = data.objects[data.numObjects].name
		end

		data.objects[data.numObjects].showProduktPerHour = Utils.getNoNil(getUserAttribute(object.nodeId, "showProduktPerHour"), true)
		if data.objects[data.numObjects].showProduktPerHour then
			data.objects[data.numObjects].produktPerHour = object.ProduktPerHour
		end

		data.objects[data.numObjects].rohstoffe = {}
		data.objects[data.numObjects].produkte = {}


		local i = 0
		for k, v in pairs(object.Rohstoffe) do
			if v.empty == nil and not v.empty then
				data.objects[data.numObjects].rohstoffe[i] = {}
				data.objects[data.numObjects].rohstoffe[i].name = v.nameL
				data.objects[data.numObjects].rohstoffe[i].fillLevel = v.fillLevel
				data.objects[data.numObjects].rohstoffe[i].capacity = v.capacity
				data.objects[data.numObjects].rohstoffe[i].id = v.id -- [UC] Added for UnitConvert Support
				i = i + 1
			end
		end
		i = 0
		for k, v in pairs(object.Produkte) do
			if v.empty == nil and not v.empty then
				data.objects[data.numObjects].produkte[i] = {}
				data.objects[data.numObjects].produkte[i].name = v.nameL
				data.objects[data.numObjects].produkte[i].fillLevel = v.fillLevel
				data.objects[data.numObjects].produkte[i].capacity = v.capacity
				data.objects[data.numObjects].produkte[i].id = v.id -- [UC] Added for UnitConvert Support
				i = i + 1
			end
		end
		if object.isOnCreate then
			local pic = getUserAttribute(object.nodeId, "tabletIconName") or getUserAttribute(object.nodeId, "tabletName")
			if pic ~= nil then
				local file = g_currentMission.baseDirectory.."farmingTabletHuds/"..pic..".dds"
				local fileFarmingTablet_App_FactoryExtension = FarmingTablet_App_FactoryExtension.dir.."farmingTabletHuds/"..pic..".dds"
				if fileExists(file) then
					pic = createImageOverlay(file)
				elseif fileExists(fileFarmingTablet_App_FactoryExtension) then
					pic = createImageOverlay(fileFarmingTablet_App_FactoryExtension)
				else
					pic = g_farmingTablet.button.getOverlayByModus_name("image", "emptyPic").id
				end
			else
				pic = g_farmingTablet.button.getOverlayByModus_name("image", "emptyPic").id
			end
			data.objects[data.numObjects].pic = pic
		else
			if fileExists(object.baseDirectory.."modDesc.xml") then
				local xml = loadXMLFile("modDesc", object.baseDirectory.."modDesc.xml", "modDesc")
				data.objects[data.numObjects].pic = createImageOverlay(object.baseDirectory .. getXMLString(xml, "modDesc.iconFilename"))
			end
		end
	end
	FarmingTablet_App_FactoryExtension.data = data
end
