--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert Farming Tablet [Horsch] AddOn
## Description: UnitConvert AddOn for handling FarmingTablet texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/05/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "FarmingTablet [Horsch Management]"
ucMod.addOns[ucAddOnIndex].source	= "FarmingTablet_App_Horsch"
ucMod.addOns[ucAddOnIndex].version	= "1.1"

local ucFarmingTabletHorsch = ucMod.addOns[ucAddOnIndex]
local FarmingTablet_App_Horsch = nil

function ucFarmingTabletHorsch:initialize()
	local target = ucFarmingTabletHorsch.target
	if target then
		target.draw_Horsch_fields = Utils.overwrittenFunction(target.draw_Horsch_fields, ucFarmingTabletHorsch.draw_Horsch_fields)

		FarmingTablet_App_Horsch = target
		return true
	end
end

function ucFarmingTabletHorsch:draw_Horsch_fields(superFunc)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "horschLogo").id, 0.09, 0.733, 0.12, 0.067)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "fieldsHud").id, 0.23, 0.733, 0.0377, 0.067)
	setTextColor(g_farmingTablet.utils.getActiveAppTextColor())
	local m = math.min(FarmingTablet_App_Horsch.numFieldToList - 1, 5)
	for i  =0, m do
		renderText(0.12, 0.555 - (i * 0.061), 0.025, FarmingTablet_App_Horsch.data[i + FarmingTablet_App_Horsch.menueState].name)
	end

	if getTableLength(FarmingTablet_App_Horsch.data) == 0 then
		return
	end
	local field = FarmingTablet_App_Horsch.data[FarmingTablet_App_Horsch.menueState + FarmingTablet_App_Horsch.activeField - 1]

	-- [UC] Get Area Unit
	local ucAreaUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)

	-- [UC] Convert Field Area
	local ucFieldArea = field.fieldArea
	ucFieldArea = ucMod:convertValue(ucFieldArea, ucAreaUnit.index, false)
	ucFieldArea = numToStr(ucFieldArea, ucAreaUnit.precision, "ceil", true, false)

	setTextBold(true)
	renderText(0.3, 0.752, 0.03, field.name)
	setTextBold(false)

	-- [UC] Display Converted Field Area
	renderText(0.6, 0.752, 0.024, string.format("%s: %s %s", g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields02"), ucFieldArea, ucAreaUnit.nameI18N))
	renderText(0.75, 0.752, 0.024, string.format("%s: ", g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields03")) .. g_i18n:formatNumber(field.fieldPrice, 0) .. " " .. g_i18n:getCurrencySymbol(true))

	--renderText(0.6, 0.752, 0.024, string.format("%s: %.2f %s", g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields02"), tostring(field.fieldArea), g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "ha")))
	--renderText(0.75, 0.752, 0.024, string.format("%s: ", g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields03")) .. g_i18n:formatNumber(field.fieldPrice, 0) .. " €")
	renderText(0.6, 0.15, 0.02, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "infoUsage"))

	renderOverlay(FarmingTablet_App_Horsch.pda, 0.6, 0.2, 0.3, 0.533)

	local x, _, z = unpack(field.pos)
	x = 0.75 + (0.3 / FarmingTablet_App_Horsch.baseMapPDAwidth * x) - 0.0025
	z = 0.4665 + (0.533 / FarmingTablet_App_Horsch.baseMapPDAwidth * -z) - 0.005
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "on").id, x, z, 0.005, 0.01)

	local i = 0

	if FarmingTablet_App_Horsch.showIndex01 == 1 then
		setTextBold(true)
		renderText(0.38, 0.69, 0.028, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields04"))
		setTextBold(false)

		local numEntrys = getTableLength(g_currentMission.fruits)
		local a = 0.025
		local b = 0.019
		if numEntrys > 15 then
			a = 0.022
			b = 0.017
		end

		for index, entry in pairs(g_currentMission.fruits) do
			local desc = FruitUtil.fruitIndexToDesc[index]
			if desc.allowsSeeding then
				renderText(0.32, 0.62 - (i * a), b, FillUtil.fillTypeIndexToDesc[FruitUtil.fruitTypeToFillType[desc.index]].nameI18N)

				-- [UC] Convert Seed Usage
				local ucSeedUsage = (field.fieldArea * 10000) * desc.seedUsagePerSqm
				local ucSeedUnit = ucMod:getFillUnit(desc.index, ucMod.UNIT_SET_STORAGE)
				ucSeedUsage = ucMod:convertValue(ucSeedUsage, ucSeedUnit.index, FillUtil.FILLTYPE_SEEDS)
				ucSeedUsage = numToStr(ucSeedUsage, ucSeedUnit.precision, "ceil", true, false)
				renderText(0.45, 0.62 - (i * a), b, string.format("%s %s", ucSeedUsage, ucSeedUnit.nameI18N))

				--renderText(0.45, 0.62 - (i * a), b, string.format("%.f %s", (field.fieldArea * 10000) * desc.seedUsagePerSqm, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "liter")))
				i = i + 1
			end
		end
	elseif FarmingTablet_App_Horsch.showIndex01 == 2 then
		setTextBold(true)
		renderText(0.38, 0.69, 0.028, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "fields05"))
		setTextBold(false)

		for _, desc in pairs(Sprayer.sprayTypeIndexToDesc) do
			local index = FillUtil.fillTypeNameToInt[desc.name]
				-- [UC] Convert Spray Usage
				local ucSprayUsage = desc.litersPerSecond * field.fieldArea * 36000
				local ucSprayFillType = FillUtil.fillTypeIndexToDesc[Sprayer.sprayTypeToFillType[desc.index]]
				local ucSprayUnit = ucMod:getFillUnit(ucSprayFillType.index, ucMod.UNIT_SET_STORAGE)
				ucSprayUsage = ucMod:convertValue(ucSprayUsage, ucSprayUnit.index, ucSprayFillType.index)
				ucSprayUsage = numToStr(ucSprayUsage, ucSprayUnit.precision, "ceil", true, false)
				renderText(0.32, 0.62 - (i * 0.03), 0.022, ucSprayFillType.nameI18N)
				renderText(0.45, 0.62 - (i * 0.03), 0.022, string.format("%s %s", ucSprayUsage, ucSprayUnit.nameI18N))

				--renderText(0.32, 0.62-(i*0.03), 0.022, FillUtil.fillTypeIndexToDesc[Sprayer.sprayTypeToFillType[desc.index]].nameI18N)
				--local liter = desc.litersPerSecond * field.fieldArea * 36000
				--renderText(0.45, 0.62-(i*0.03), 0.022, string.format("%.f %s", liter, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_Horsch.name, "liter")))
				i = i + 1
		end
	end
	g_farmingTablet.button.drawButtonByCategory("Horsch_FIELDS")
end
