--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert Farming Tablet [StorageHouse] AddOn
## Description: UnitConvert AddOn for handling FarmingTablet texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/05/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "FarmingTablet [StorageHouse]"
ucMod.addOns[ucAddOnIndex].source	= "FarmingTablet_App_StorageHouse"
ucMod.addOns[ucAddOnIndex].version	= "1.0.0.0"

local ucFarmingTabletStorageHouse = ucMod.addOns[ucAddOnIndex]
local FarmingTablet_App_StorageHouse = nil

function ucFarmingTabletStorageHouse:initialize()
	local target = ucFarmingTabletStorageHouse.target
	if target then
		target.draw_StorageHouse_overview = Utils.overwrittenFunction(target.draw_StorageHouse_overview, ucFarmingTabletStorageHouse.draw_StorageHouse_overview)
		target.getStorageHouseInfo = Utils.overwrittenFunction(target.getStorageHouseInfo, ucFarmingTabletStorageHouse.getStorageHouseInfo)

		FarmingTablet_App_StorageHouse = target
		return true
	end
end

function ucFarmingTabletStorageHouse:draw_StorageHouse_overview(superFunc)
	local storedObjects = g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageHouse.name, "storedObjects")
	local objectCapacity = g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageHouse.name, "objectCapacity")
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "backScreen_storageHouse").id, 0.008, 0.0163, 1, 0.964)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "backScreen_DarkAlpha").id, 0.36, 0.14, 0.55, 0.67)
	setTextColor(0.9801, 0.07, 0.01, 1)
	setTextAlignment(RenderText.ALIGN_CENTER)
	renderText(0.63, 0.775, 0.022, storedObjects)
	renderText(0.8, 0.775, 0.022, objectCapacity)

	g_farmingTablet.button.drawButtonByCategory("STORAGEHOUSE_PAGE_FRONT")
	g_farmingTablet.button.drawButtonByCategory("STORAGEHOUSE_PAGE_MENUEITEMS")
	g_farmingTablet.button.drawButtonByCategory("STORAGEHOUSE_PAGE_MENUPRODUCTS")

	setTextColor(1, 1, 1, 1)
	if FarmingTablet_App_StorageHouse.data.objects[FarmingTablet_App_StorageHouse.activeSilo] ~= nil then
		local object = FarmingTablet_App_StorageHouse.data.objects[FarmingTablet_App_StorageHouse.activeSilo]
		for i = 0, math.min(table.getn(object.tabletData), 5) do
			local v = object.tabletData[i + FarmingTablet_App_StorageHouse.productState - 1]
			setTextAlignment(RenderText.ALIGN_CENTER)
			pic = createImageOverlay(v.image)
			renderOverlay(pic, 0.71, 0.67 - (i * 0.1), 0.06, 0.107)
			renderText(0.46, 0.71 - (i * 0.1), 0.03, v.name)
			setTextAlignment(RenderText.ALIGN_CENTER)
			renderText(0.63, 0.71 - (i * 0.1), 0.03, tostring(string.format("%.f / %s", v.stored, v.capacity)))

			-- [UC] Convert FillType
			local ucFillUnit = ucMod:getFillUnit(v.fillType, ucMod.UNIT_SET_STORAGE)
			local ucFillLevel = v.nextObjectSize
			ucFillLevel = ucMod:convertValue(ucFillLevel, ucFillUnit.index, v.fillType)
			ucFillLevel = numToStr(ucFillLevel, ucFillUnit.precision, "ceil", true, false)
			renderText(0.85, 0.71 - (i * 0.1), 0.03, string.format("%s %s", ucFillLevel, ucFillUnit.nameI18NShort))

			--renderText(0.85, 0.71 - (i * 0.1), 0.03, tostring(string.format("%d", v.nextObjectSize)) .. "  l")
		end
	end

	local i = FarmingTablet_App_StorageHouse.menueState
	local h = 1
	for j = FarmingTablet_App_StorageHouse.menueState, FarmingTablet_App_StorageHouse.menueState + 5 do
		if j > table.getn(FarmingTablet_App_StorageHouse.data.objects) then
			break
		end
		if i == FarmingTablet_App_StorageHouse.activeSilo then
			setTextAlignment(RenderText.ALIGN_CENTER)
			setTextBold(true)
			setTextColor(0.9801, 0.07, 0.01, 1)
			renderText(0.21, (0.625 - ((h - 1) * 0.074)), 0.03, FarmingTablet_App_StorageHouse.data.objects[i].name)
			setTextBold(false)
			setTextColor(1, 1, 1, 1)
		else
			renderText(0.21, (0.625 - ((h - 1) * 0.074)), 0.03, FarmingTablet_App_StorageHouse.data.objects[i].name)
		end
		h = h + 1
		i = i + 1
	end
end

function ucFarmingTabletStorageHouse:getStorageHouseInfo(superFunc)
	local data = {}
	data.numObjects = 0
	data.objects = {}
	if g_currentMission.onCreateLoadedObjects ~= nil then
		for _, onCreateObject in pairs(g_currentMission.onCreateLoadedObjects) do
			if onCreateObject.StorageHouseDirtyFlag then
				data.numObjects = data.numObjects + 1
				local name = onCreateObject.objectStorageName
				data.objects[data.numObjects] = {}
				data.objects[data.numObjects].name = name
				data.objects[data.numObjects].tabletData = {}
				local i = 0
				for k, v in pairs(onCreateObject.storage) do
					data.objects[data.numObjects].tabletData[i] = {}
					data.objects[data.numObjects].tabletData[i].name		= onCreateObject.objectData[k].uiNameId
					data.objects[data.numObjects].tabletData[i].image		= onCreateObject.objectData[k].uiImageId
					data.objects[data.numObjects].tabletData[i].fillType	= onCreateObject.objectData[k].objectFillType -- [UC] Added for UnitConvert Support
					data.objects[data.numObjects].tabletData[i].stored		= table.getn(v.storedItems)
					local objectSize = 0
					for _, nextObject in pairs(v.storedItems) do
						if nextObject ~= nil then
							objectSize = nextObject
						end
					end
					data.objects[data.numObjects].tabletData[i].nextObjectSize = objectSize
					data.objects[data.numObjects].tabletData[i].capacity = v.capacity
					i = i + 1
				end
			end
		end
	end
	if g_currentMission.ownedItems ~= nil then
		for _, items in pairs(g_currentMission.ownedItems) do
			for _, item in pairs(items.items) do
				if item.StorageHouseDirtyFlag then
					data.numObjects = data.numObjects + 1
					local name = item.objectStorageName
					data.objects[data.numObjects] = {}
					data.objects[data.numObjects].name = name
					data.objects[data.numObjects].tabletData = {}
					local i = 0
					for k, v in pairs(item.storage) do
						data.objects[data.numObjects].tabletData[i] = {}
						data.objects[data.numObjects].tabletData[i].name		= item.objectData[k].uiNameId
						data.objects[data.numObjects].tabletData[i].image		= item.objectData[k].uiImageId
						data.objects[data.numObjects].tabletData[i].fillType	= item.objectData[k].objectFillType -- [UC] Added for UnitConvert Support
						data.objects[data.numObjects].tabletData[i].stored		= table.getn(v.storedItems)
						local objectSize = 0
						for _, nextObject in pairs(v.storedItems) do
							if nextObject ~= nil then
								objectSize = nextObject
							end
						end
						data.objects[data.numObjects].tabletData[i].nextObjectSize = objectSize
						data.objects[data.numObjects].tabletData[i].capacity = v.capacity
						i = i + 1
					end
				end
			end
		end
	end
	return data
end

