--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert Farming Tablet [StorageOverview] AddOn
## Description: UnitConvert AddOn for handling FarmingTablet texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/05/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "FarmingTablet [StorageOverview]"
ucMod.addOns[ucAddOnIndex].source	= "FarmingTablet_App_StorageOverview"
ucMod.addOns[ucAddOnIndex].version	= "1.2.0.1"

local ucFarmingTabletStorageOverview = ucMod.addOns[ucAddOnIndex]
local FarmingTablet_App_StorageOverview = nil

function ucFarmingTabletStorageOverview:initialize()
	local target = ucFarmingTabletStorageOverview.target
	if target then
		-- Create FillType NameI18N Table (For Reference)
		if not target.fillTypeNameI18NToDesc then
			target.fillTypeNameI18NToDesc = {}
			for _, fillType in pairs(FillUtil.fillTypeIndexToDesc) do
				if fillType.nameI18N then
					target.fillTypeNameI18NToDesc[fillType.nameI18N] = fillType
				end
			end
		end

		target.draw_StorageOverview_page = Utils.overwrittenFunction(target.draw_StorageOverview_page, ucFarmingTabletStorageOverview.draw_StorageOverview_page)

		FarmingTablet_App_StorageOverview = target
		return true
	end
end

function ucFarmingTabletStorageOverview:draw_StorageOverview_page(superFunc)
	renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "backScreenStorageOverview").id, 0.008, 0.0163, 1, 0.964)
	g_farmingTablet.button.drawButtonByCategory("STORAGEOVERVIEW_PAGE")

	FarmingTablet_App_StorageOverview.data = FarmingTablet_App_StorageOverview:getData()

	if FarmingTablet_App_StorageOverview.data.objects[FarmingTablet_App_StorageOverview.activeSilo] ~= nil then
		if table.getn(FarmingTablet_App_StorageOverview.data.objects[FarmingTablet_App_StorageOverview.activeSilo].tabletData) > 5 then
			g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_up").visible = true
			g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_down").visible = true
		else
			g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_up").visible = false
			g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_down").visible = false
		end
	else
		g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_up").visible = false
		g_farmingTablet.button.getButtonByApp(FarmingTablet_App_StorageOverview.name, "Product_down").visible = false
	end

	if FarmingTablet_App_StorageOverview.data.objects[FarmingTablet_App_StorageOverview.activeSilo] ~= nil then
		local object = FarmingTablet_App_StorageOverview.data.objects[FarmingTablet_App_StorageOverview.activeSilo]
		FarmingTablet_App_StorageOverview:doUpdates(object)

		-- [UC] Create FillTypes Table
		if not object.fillTypes then
			object.fillTypes = {}
			for _, data in pairs(object.tabletData) do
				local fillType = FarmingTablet_App_StorageOverview.fillTypeNameI18NToDesc[data.fillType]
				if fillType then
					table.insert(object.fillTypes, fillType.index)
				end
			end
		end

		-- Silo Location
		renderOverlay(FarmingTablet_App_StorageOverview.pda, 0.122, 0.199, 0.27, 0.48)
		local x, _, z = getWorldTranslation(object.node)
		x = 0.257 + (0.27 / FarmingTablet_App_StorageOverview.baseMapPDAwidth * x) - 0.0025
		z = 0.439 + (0.48 / FarmingTablet_App_StorageOverview.baseMapPDAwidth * -z) - 0.005

		if object.id == FarmingTablet_App_StorageOverview.activeSilo then
			renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "onStorageOverview").id, x, z, 0.005, 0.01)
		end

		local text = string.format("%s / %s  %s", FarmingTablet_App_StorageOverview.activeSilo, table.getn(FarmingTablet_App_StorageOverview.data.objects), object.name)
		local x = 0.649 - getTextWidth(0.032, text) / 2

		local fixedTotal = 0
		if object.totalWorth ~= nil then
			fixedTotal = object.totalWorth
		else
			fixedTotal = 0
		end

		local factor = g_currentMission.missionInfo.buyPriceMultiplier * object.costPerDay

		local cost = 0
		for _, lvl in pairs(object.tabletData) do
            cost = cost + lvl.level * factor
        end

		setTextColor(0.106, 0.240, 0.038, 1)
		setTextBold(true)

		renderText(x, 0.71, 0.032, text)

		setTextColor(0.128, 0.128, 0.128, 1)
		setTextBold(false)

		renderText(0.15, 0.732, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "silo"))
		renderText(0.245, 0.732, 0.021, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "bestSell"))

		setTextColor(1, 1, 1, 1)

		-- [UC] Convert Capacity
		local ucCapacity = object.capacity
		local ucFillUnit = ucMod:getFillUnit(object.fillTypes, ucMod.UNIT_SET_STORAGE)
		local ucBestFillType = ucMod:getHeaviestFillType(object.fillTypes)
		ucCapacity = ucMod:convertValue(ucCapacity, ucFillUnit.index, ucBestFillType)
		ucCapacity = numToStr(ucCapacity, ucFillUnit.precision, "ceil", true, false)
		renderText(0.43, 0.66, 0.026, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "capacity") .. ucCapacity .. " " .. ucFillUnit.nameI18NShort)

		--renderText(0.43, 0.66, 0.026, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "capacity") .. g_i18n:formatNumber(object.capacity, 0) .. " l")
		renderText(0.43, 0.59, 0.028, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "headerOne"))

		setTextAlignment(RenderText.ALIGN_RIGHT)

		renderText(0.86, 0.59, 0.028, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "headerTwo"))
		renderText(0.86, 0.195, 0.028, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "totalValue") .. g_i18n:formatMoney(fixedTotal, 0))
		renderText(0.86, 0.66, 0.026, g_farmingTablet.utils.getTextByAppName(FarmingTablet_App_StorageOverview.name, "costPerDay") .. g_i18n:formatMoney(cost, 0))

		setTextAlignment(RenderText.ALIGN_LEFT)

		FarmingTablet_App_StorageOverview.mouseOverActive = false
		for i = 0, math.min(table.getn(object.tabletData), 5) do
			local v = object.tabletData[i + FarmingTablet_App_StorageOverview.productState - 1]

			if FarmingTablet_App_StorageOverview:buttonHover(FarmingTablet_App_StorageOverview.mousePosition[1], FarmingTablet_App_StorageOverview.mousePosition[2], 0.42, 0.89, 0.510 - (i * 0.05), 0.540 - (i * 0.05)) then
				if v.sellPoint ~= nil then
					FarmingTablet_App_StorageOverview.mouseOverActive = true
					if not FarmingTablet_App_StorageOverview.mapPointMarked[i] then
						renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "mouseButton").id, 0.43, (0.510-(i * 0.05)), 0.0153, 0.03)
					end
					if InputBinding.hasEvent(InputBinding.TARGET_SELL_POINT) then
						FarmingTablet_App_StorageOverview:onClickPrice(v)
					end
				end

				if v.storedValue ~= nil then
					renderText(0.46, (0.515 - (i * 0.05)), 0.03, v.bestLocation)
					setTextAlignment(RenderText.ALIGN_RIGHT)
					renderText(0.85, (0.515 - (i * 0.05)), 0.03, g_i18n:formatMoney(v.storedValue))
					setTextAlignment(RenderText.ALIGN_LEFT)
				end
			else
				renderText(0.46, (0.515 - (i * 0.05)), 0.03, v.fillType)
				setTextAlignment(RenderText.ALIGN_RIGHT)

				-- [UC] Convert Fill Level
				local ucFillLevel = v.level
				local ucFillType = FarmingTablet_App_StorageOverview.fillTypeNameI18NToDesc[v.fillType].index
				local ucFillUnit = ucMod:getFillUnit(ucFillType, ucMod.UNIT_SET_STORAGE)
				ucFillLevel = ucMod:convertValue(ucFillLevel, ucFillUnit.index, ucFillType)
				ucFillLevel = numToStr(ucFillLevel, ucFillUnit.precision, "ceil", true, false)
				renderText(0.85, (0.515 - (i * 0.05)), 0.03, ucFillLevel .. " " .. ucFillUnit.nameI18NShort)

				--renderText(0.85, (0.515 - (i * 0.05)), 0.03, g_i18n:formatNumber(v.level, 0) .. " l")
				setTextAlignment(RenderText.ALIGN_LEFT)
			end

			-- Sell Point Location.
			if FarmingTablet_App_StorageOverview.mapPointMarked[i] then
				local pulse = Utils.clamp(math.cos(9.0 * (g_currentMission.time / 2000)), 0, 1)
				if v.sellPoint ~= nil then
					local x, _, z = getWorldTranslation(v.sellPoint.rootNode)
					x = 0.257 + (0.27 / FarmingTablet_App_StorageOverview.baseMapPDAwidth * x) - 0.0025
					z = 0.439 + (0.48 / FarmingTablet_App_StorageOverview.baseMapPDAwidth * -z) - 0.005
					if pulse > 0 then
						renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "onSellPoint").id, x, z, 0.005, 0.01)
					end
					renderOverlay(g_farmingTablet.button.getOverlayByModus_name("image", "onSellPointMenu").id, 0.43, (0.510 - (i * 0.05)), 0.0153, 0.03)
				end
			end
		end
	else
		renderText(0.46, 0.721, 0.026, "Keine Silos gefunden / No Silos Found")
	end
end
