--[[
## Interface: FS17 v1.5.3.1
## Title: UnitConvert GPSMod AddOn
## Description: UnitConvert AddOn for handling GPSMod texts.
## Author: ThundR75
## Version: 1.0.0.0
## Date: 09/04/2018

## ChangeLog:
	Version 1.0.0.0
		- Initial release
--]]

local ucAddOnIndex = getTableLength(ucMod.addOns) + 1
ucMod.addOns[ucAddOnIndex] = {}
ucMod.addOns[ucAddOnIndex].name		= "GPSMod"
ucMod.addOns[ucAddOnIndex].source	= "GPS"
ucMod.addOns[ucAddOnIndex].version	= "5.01"

local ucGPSMod = ucMod.addOns[ucAddOnIndex]
local GPS = nil

function ucGPSMod:initialize()
	local target = ucGPSMod.target
	if target then
		target.draw = Utils.overwrittenFunction(target.draw, ucGPSMod.draw)

		GPS = target
		return true
	end
end

function ucGPSMod:draw(superFunc)
	if not self.GPSActive then
		g_currentMission:addHelpButtonText(Steerable.turnOn, InputBinding.GPS_OnOff, nil, GS_PRIO_LOW)
	else
		g_currentMission:addHelpButtonText(Steerable.turnOff, InputBinding.GPS_OnOff, nil, GS_PRIO_LOW)
		g_currentMission:addHelpButtonText(Steerable.GPS_InfoMode, InputBinding.GPS_InfoMode, nil, GS_PRIO_LOW)

		if GPS.Store[self.GPS_storeSlot] ~= nil then
			local slotName = GPS.Store[self.GPS_storeSlot].GPS_slotName
			if slotName ~= nil and slotName ~= "" then
				g_currentMission:addExtraPrintText(string.format(Steerable.GPS_NAME, self.typeDesc) .. "("..tostring(self.GPS_storeSlot).."): "..tostring(GPS.Store[self.GPS_storeSlot].GPS_slotName))
			end
		end
	end

	if self.GPSshowMode < 3 and self.GPSActive then
		if true then
			-- [UC] Get Length Unit
			local ucLengthUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_LENGTH)

			local isPDA = false --LS15
			if isPDA and self.GPSshowMode > 1 then
				self.GPSshowMode = 1
			end
			for mode=1, self.GPSshowMode do
				local hudField = GPS.GPS_HUDfields[mode]

				if hudField.showOnPDA then
					if isPDA then
						hudField.background:setPosition(hudField.pdaX, hudField.pdaY)
					else
						hudField.background:setPosition(hudField.normalX, hudField.normalY)
					end
				end

				hudField.background:render()

				for o, ov in pairs(GPS.GPS_HUDfields[mode].boolOverlays) do
					if ov.Ncond > 0 then
						local c1 = self[ov.cond1]
						local c2 = true
						if ov.cond1Inv then
							c1 = not c1
						end
						if ov.Ncond > 1 then
							c2 = self[ov.cond2]
							if ov.cond2Inv then
								c2 = not c2
							end
						end
						if c1 and c2 then
							if isPDA then
								if hudField.showOnPDA then
									ov.overlay:setPosition(hudField.pdaX, hudField.pdaY)
									ov.overlay:render()
								end
							else
								if hudField.showOnPDA then
									ov.overlay:setPosition(hudField.normalX, hudField.normalY)
								end
								ov.overlay:render()
							end
						end
					end
				end

				setTextAlignment(RenderText.ALIGN_LEFT)
				setTextBold(false)
				setTextColor(0.8, 0.8, 0.8, 0.9)

				for t, txt in pairs(GPS.GPS_HUDfields[mode].TXTfields) do
					local doTxt = true
					if txt.boolean ~= nil then
						if txt.invert then
							doTxt = not self[txt.boolean]
						else
							doTxt = self[txt.boolean]
						end
					end

					if doTxt then
						if txt.alignCenter then
							setTextAlignment(RenderText.ALIGN_CENTER)
						else
							setTextAlignment(RenderText.ALIGN_LEFT)
						end
						local x = txt.xCoord
						local y = txt.yCoord
						if isPDA then
							y = Utils.getNoNil(txt.yPDA,y)
						end

						setTextBold(false)
						if txt.boldBoolean ~= nil then
							if self[txt.boldBoolean] then
								setTextBold(true)
							end
						end

						-- [UC] Convert Values
						local ucIsConverted = false
						local ucVarStr = txt.varStr
						local ucValue = self[ucVarStr]

						if ucVarStr == "GPS_HUD_distanceStr"
						or ucVarStr == "GPS_HUD_OffsetStr"
						or ucVarStr == "GPSWidth"
						or ucVarStr == "GPSautoStopDistance"
						then
							ucValue = strToNum(ucValue)
							if getIsNumber(ucValue) then
								ucValue = ucMod:convertValue(ucValue, ucLengthUnit.index, false)
								ucValue = numToStr(ucValue, 1, "round", true, false) -- [UC] Precision must be 1 or less to avoid text overlap
								renderText(x, y, txt.size, ucValue)
								ucIsConverted = true
							end
						end

						if not ucIsConverted then
							renderText(x, y, txt.size, string.format(txt.str, self[txt.varStr]))
						end
					end

					setTextAlignment(RenderText.ALIGN_LEFT)
					setTextBold(false)
				end

				for b, button in pairs(GPS.GPS_HUDfields[mode].buttons) do
					if button.pressedOverlay~= nil then
						if InputBinding.isPressed(button.binding) then
							if InputBinding.isPressed(InputBinding.GPS_adjustCourseModifier) == button.modifier then
								button.pressedOverlay:render()
							end
						end
					end
				end
			end
		end
	end
end