﻿--[[
## Title:  UnitConvert
## Author: ThundR75
--]]

-- [DECLARATIONS] *************************************************************************************************************************************************************************************

UnitConvert = {}
getfenv(0)["g_unitConvert"] = UnitConvert
ucMod = g_unitConvert

-- Utility variables (for external scripts and error handling)
globalCaller = ucMod
globalErrorText = "UC_error_internal"

local modName = g_currentModName
ucMod.desc = ModsUtil.modNameToMod[modName]
ucMod.title = ucMod.desc.title
ucMod.version = ucMod.desc.version
ucMod.modDir = g_currentModDirectory

ucMod.debugLvl = 0
ucMod.isDev = false
ucMod.isBeta = false

-- Load Extra Scripts
source(ucMod.modDir.."extras/utils.lua")

-- Load Gui Scripts
source(ucMod.modDir.."gui/ucMenu.lua")

-- Load Third Party AddOn Scripts
ucMod.addOns = {}
ucMod.addOnDir = ucMod.modDir.."addons/"
source(ucMod.addOnDir.."ucGPSMod.lua")
source(ucMod.addOnDir.."ucFarmingTablet.lua")
source(ucMod.addOnDir.."ucFarmingTabletAnimalStatus.lua")
source(ucMod.addOnDir.."ucFarmingTabletBunkerSiloOverview.lua")
source(ucMod.addOnDir.."ucFarmingTabletStorageOverview.lua")
source(ucMod.addOnDir.."ucFarmingTabletFactoryExtension.lua")
source(ucMod.addOnDir.."ucFarmingTabletFarmSiloSystem.lua")
source(ucMod.addOnDir.."ucFarmingTabletStorageHouse.lua")
source(ucMod.addOnDir.."ucFarmingTabletHorsch.lua")
source(ucMod.addOnDir.."ucDriveControl.lua")

-- [GLOBAL FUNCTIONS] *********************************************************************************************************************************************************************************

function UnitConvert:getIsFruitTypeValid(fruitType)
-- This function checks if a fruitType is valid
	--[fruitType]: FruitType to check (by index or name)

-- Returns true if fruitType is valid

	if fruitType then
		if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
			if xAssert(Utils.getNoNil(FruitUtil.fruitIndexToDesc[fruitType], FruitUtil.fruitTypes[fruitType])) then
				return true
			end
		end
	end
end

function UnitConvert:getIsFillTypeValid(fillType)
-- This function checks if a fillType is valid
	-- [fillType]: FillType to check (by index or name)

-- Returns true if fillType is valid

	if fillType then
		if fillType ~= FillUtil.FILLTYPE_UNKNOWN then
			if xAssert(Utils.getNoNil(FillUtil.fillTypeIndexToDesc[fillType], FillUtil.fillTypeNameToDesc[fillType])) then
				return true
			end
		end
	end
end

function UnitConvert:getIsFillTypeCategoryValid(fillTypeCategory)
-- This function check if a fillType category is valid
	-- [fillTypeCategory]: FillType category to check (by index or name)

-- Returns true if fillType category is valid

	if fillTypeCategory then
		fillTypeCategory = Utils.getNoNil(FillUtil.fillTypeCategoryNameToInt[fillTypeCategory], fillTypeCategory)
		if fillTypeCategory ~= FillUtil.FILLTYPE_CATEGORY_UNKNOWN then
			if xAssert(FillUtil.fillTypeCategoryIntToName[fillTypeCategory]) then
				return true
			end
		end
	end
end

function UnitConvert:getFillTypesByCategories(categories)
-- This function retrieves all fillTypes belonging to the specified fillType categories
	-- [categories]: Giants fillType categories to check (by index, list or table of categories)

-- Returns a table containing all fillTypes belonging to above categories

	if not categories then
		raiseError()
		return
	end

	local fillTypes = {}

	if getIsNumber(categories) then
		categories = {categories}
	else
		if not getIsTable(categories) then
			categories = splitString(" ", categories)
		end
	end

	for _, category in pairs(categories) do
		if ucMod:getIsFillTypeCategoryValid(category) then
			category = xAssert(Utils.getNoNil(tonumber(category), FillUtil.fillTypeCategoryNameToInt[category]))
			local fillTypeList = FillUtil.categoryToFillTypes[category]
			if fillTypeList then
				for fillType, isValid in pairs(fillTypeList) do
					if isValid and ucMod:getIsFillTypeValid(fillType) and not fillTypes[fillType] then
						table.insert(fillTypes, fillType)
					end
				end
			end
		end
	end

	if getTableLength(fillTypes) > 0 then return fillTypes end
end

function UnitConvert:getDefaultUnitTypes()
-- This function simply returns the default unit types for a new fillTypeGroup

	local retTbl = {}
	retTbl[ucMod.UNIT_TYPE_VOLUME] = true
	retTbl[ucMod.UNIT_TYPE_WEIGHT] = true

	return retTbl
end

function UnitConvert:getDefaultFillTypeGroup(fillType)
-- This function does its best to find which fillType group a fillType belongs to
	-- [fillType]: FillType to check (by index or name)

-- Returns the index of the default fillType group the fillType belongs to (defaults to "bulk")

	if not ucMod:getIsFillTypeValid(fillType) then
		raiseError()
		return
	end

	fillType = Utils.getNoNil(FillUtil.fillTypeNameToDesc[fillType], FillUtil.fillTypeIndexToDesc[fillType])

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_ANIMAL][fillType.index] then
		if ucMod.FILLTYPE_GROUP_ANIMAL then
			return ucMod.FILLTYPE_GROUP_ANIMAL
		end
	end

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_PIECE][fillType.index] then
		if ucMod.FILLTYPE_GROUP_PIECE then
			return ucMod.FILLTYPE_GROUP_PIECE
		end
	end

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_LIQUID][fillType.index]
	or FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_SPRAYER][fillType.index]
	or FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_SLURRYTANK][fillType.index]
	then
		if ucMod.FILLTYPE_GROUP_LIQUID then
			return ucMod.FILLTYPE_GROUP_LIQUID
		end
	end

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_COMBINE][fillType.index] then
		if ucMod.FILLTYPE_GROUP_GRAIN then
			return ucMod.FILLTYPE_GROUP_GRAIN
		end
	end

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_SPREADER][fillType.index]
	or FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_MANURESPREADER][fillType.index]
	then
		if ucMod.FILLTYPE_GROUP_FERTILIZER then
			return ucMod.FILLTYPE_GROUP_FERTILIZER
		end
	end

	if FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_WINDROW][fillType.index]
	or FillUtil.categoryToFillTypes[FillUtil.FILLTYPE_CATEGORY_FORAGEWAGON][fillType.index]
	then
		if ucMod.FILLTYPE_GROUP_WINDROW then
			return ucMod.FILLTYPE_GROUP_WINDROW
		end
	end

	local fruitType = FruitUtil.fillTypeToFruitType[fillType.index]
	if fruitType then
		local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType]
		if fruitDesc.preparedGrowthState and fruitDesc.preparedGrowthState > 0 then
			if ucMod.FILLTYPE_GROUP_ROOTCROP then
				return ucMod.FILLTYPE_GROUP_ROOTCROP
			end
		end

		local fillTypeName = string.upper(fillType.name)
		if fillTypeName then
			if string.find(fillTypeName, "DIRT")
			or string.find(fillTypeName, "CONCRETE")
			or string.find(fillTypeName, "CEMENT")
			or string.find(fillTypeName, "GRAVEL")
			or string.find(fillTypeName, "STONE")
			or string.find(fillTypeName, "ROCK")
			or string.find(fillTypeName, "SAND")
			or string.find(fillTypeName, "ASPHALT")
			then
				if ucMod.FILLTYPE_GROUP_CONSTRUCTION then
					return ucMod.FILLTYPE_GROUP_CONSTRUCTION
				end
			end
		end
	end

	return ucMod.FILLTYPE_GROUP_BULK
end

function UnitConvert:addFillTypeGroup(name)
-- This function adds a new fillType group
	-- [name]: Name of fillType group to add

-- Returns a table containing the new fillType group information (nil on failure)

	if not name or (name and not getIsString(name)) then
		raiseError()
		return
	end

	if not ucMod.fillTypeGroupsByName[name] then
		local index = getTableLength(ucMod.fillTypeGroupsByIndex) + 1
		local fillTypeGroup = {
			name		= name,
			index		= index,
			priority	= index,
			unitTypes	= ucMod:getDefaultUnitTypes(),
			units		= {}
		}

		for _, unitSet in pairs(ucMod.unitSetsByIndex) do
			local defaultUnit = ucMod:getDefaultUnit(fillTypeGroup.unitTypes, true)
			fillTypeGroup.units[unitSet.index] = defaultUnit.index
		end

		ucMod.fillTypeGroupsByIndex[fillTypeGroup.index] = fillTypeGroup
		ucMod.fillTypeGroupsByName[fillTypeGroup.name] = fillTypeGroup
		ucMod.fillTypeGroupsByPriority[fillTypeGroup.priority] = fillTypeGroup
		ucMod["FILLTYPE_GROUP_"..string.upper(fillTypeGroup.name)] = fillTypeGroup.index
		ucMod.NUM_FILLTYPE_GROUPS = getTableLength(ucMod.fillTypeGroupsByIndex)
	end

	return fillTypeGroup
end

function UnitConvert:removeFillTypeGroup(name)
-- This function removes a fillType group
	-- [name]: Name of fillTypeGroup to remove

	if not name or (name and not getIsString(name)) then
		raiseError()
		return
	end

	local fillTypeGroup = ucMod:getFillTypeGroup(name)

	if not fillTypeGroup.isDefault then
		table.remove(ucMod.fillTypeGroupsByIndex, fillTypeGroup.index)
		ucMod.fillTypeGroupsByName[fillTypeGroup.name] = nil
		ucMod["FILLTYPE_GROUP_"..string.upper(fillTypeGroup.name)] = nil
	end

	for k, v in pairs(ucMod.fillTypeGroupsByIndex) do
		v.index = k
		ucMod["FILLTYPE_GROUP_"..string.upper(v.name)] = k
	end

	ucMod.fillTypeGroupsByPriority = indexTableByKey(ucMod.fillTypeGroupsByIndex, "priority")
	ucMod.NUM_FILLTYPE_GROUPS = getTableLength(ucMod.fillTypeGroupsByIndex)

	-- Refresh fillTypes
	for _, fillType in pairs(ucMod.fillTypesByIndex) do
		if fillType.group == fillTypeGroup.index then
			fillType.group = ucMod:getDefaultFillTypeGroup(fillType.index)
		elseif fillType.group > fillTypeGroup.index then
			if fillType.group > 1 then
				fillType.group = fillType.group - 1
			else
				fillType.group = ucMod:getDefaultFillTypeGroup(fillType.index)
			end
		end
	end
end

function UnitConvert:getUnit(unit)
-- This function retrieves UnitConvert unit information
	-- [unit]: Unit to retrieve (by index or name)

-- Returns a table containing the unit information (error on failure)

	unit = xAssert(Utils.getNoNil(tonumber(unit), unit))
	return xAssert(Utils.getNoNil(ucMod.unitsByIndex[unit], ucMod.unitsByName[unit]))
end

function UnitConvert:getDefaultUnit(unitTypes, useBaseUnit)
-- This function retrieves the default unit associated with the specified unit types
	-- [unitTypes]:		Unit types to check (by index, list, or table of unit types)
	-- [useBaseUnit]:	Ignores user assigned unit if true

-- Returns a table containing the default unit information (error on failure)

	if (not getIsTable(unitTypes) and not getIsNumber(unitTypes) and not getIsString(unitTypes))
	or (getIsTable(unitTypes) and getTableLength(unitTypes) == 0)
	then
		raiseError()
		return
	end

	local unitTypesList = unitTypes
	unitTypes = {}

	if getIsNumber(unitTypesList) then
		unitTypes[unitTypesList] = true
	else
		if not getIsTable(unitTypesList) then
			unitTypesList = splitString(" ", unitTypesList)
		end

		for idx, item in pairs(unitTypesList) do
			if getIsTable(item) and item.index then
				unitTypes[item.index] = true
			elseif getIsBoolean(item) and item == true then
				unitTypes[idx] = true
			else
				unitTypes[item] = true
			end
		end
	end

	for _, unitType in ipairs(ucMod.unitTypesByPriority) do
		for unitTypeToCheck, isValid in pairs(unitTypes) do
			if isValid then
				unitTypeToCheck = ucMod:getUnit(unitTypeToCheck)
				if unitTypeToCheck.index == unitType.index then
					local retUnit = nil
					if unitTypeToCheck.index == ucMod.UNIT_TYPE_AREA then
						retUnit = ucMod.UNIT_HECTARE
					elseif unitTypeToCheck.index == ucMod.UNIT_TYPE_LENGTH then
						retUnit = ucMod.UNIT_METER
					elseif unitTypeToCheck.index == ucMod.UNIT_TYPE_PIECE then
						retUnit = ucMod.UNIT_PIECE
					elseif unitTypeToCheck.index == ucMod.UNIT_TYPE_VOLUME then
						retUnit = ucMod.UNIT_LITER
					elseif unitTypeToCheck.index == ucMod.UNIT_TYPE_WEIGHT then
						retUnit = ucMod.UNIT_KILOGRAM
					end
					if retUnit then
						if not useBaseUnit then
							retUnit = Utils.getNoNil(unitType.defaultUnit, retUnit)
						end
						return ucMod:getUnit(retUnit)
					end
				end
			end
		end
	end
	raiseError()
end

function UnitConvert:getFillUnit(fillTypes, unitSet)
-- This function retrieves the unit associated with the specified fillTypes and unitSet
	-- [fillTypes]: List of fillTypes to check (by index, list or table of fillTypes)
	-- [unitSet]:	Unit set to retrieve (by index or name)

-- Returns a table containing unit information (error on failure)

	if (not getIsTable(fillTypes) and not getIsNumber(fillTypes) and not getIsString(fillTypes))
	or (getIsTable(fillTypes) and getTableLength(fillTypes) == 0)
	or (not getIsNumber(unitSet) and not getIsString(unitSet))
	then
		raiseError()
		return
	end

	if getIsNumber(fillTypes) then
		fillTypes = {fillTypes}
	else
		if not getIsTable(fillTypes) then
			fillTypes = splitString(" ", fillTypes)
		end
	end

	unitSet = ucMod:getUnitSet(unitSet)

	for _, fillTypeGroup in ipairs(ucMod.fillTypeGroupsByPriority) do
		for _, fillType in pairs(fillTypes) do
			if ucMod:getIsFillTypeValid(fillType) then
				fillType = ucMod:getFillType(fillType)
				if fillType.group == fillTypeGroup.index then
					return ucMod:getUnit(fillTypeGroup.units[unitSet.index])
				end
			end
		end
	end

	raiseError()
end

function UnitConvert:getUnitType(unitType)
-- This function retrieves UnitConvert unitType information based on the unitType specified
	-- [unitType]: Unit type to retrieve (by index or name)

-- Returns a table containing unitType information (error on failure)

	unitType = xAssert(Utils.getNoNil(tonumber(unitType), unitType))
	return xAssert(Utils.getNoNil(ucMod.unitTypesByIndex[unitType], ucMod.unitTypesByName[unitType]))
end

function UnitConvert:getUnitSet(unitSet)
-- This function retrieves UnitConver unitSet information based on the unitSet specified
	-- [unitSet]: UnitSet to retrieve (by index or name)

-- Returns a table containing the unitSet information (error on failure)

	unitSet = xAssert(Utils.getNoNil(tonumber(unitSet), unitSet))
	return xAssert(Utils.getNoNil(ucMod.unitSetsByIndex[unitSet], ucMod.unitSetsByName[unitSet]))
end

function UnitConvert:getFillTypeGroup(fillTypeGroup)
-- This function retrieves fillType group information based on the fillType group specified
	-- [fillTypeGroup]: FillType group to retrieve (by index or name)

-- Returns a table containing fillType group information (error on failure)

	fillTypeGroup = xAssert(Utils.getNoNil(tonumber(fillTypeGroup), fillTypeGroup))
	return xAssert(Utils.getNoNil(ucMod.fillTypeGroupsByIndex[fillTypeGroup], ucMod.fillTypeGroupsByName[fillTypeGroup]))
end

function UnitConvert:getFillType(fillType)
-- This function retrieves UnitConvert fillType information based on the fillType specified
	-- [fillType]: FillType to retrieve (by index or name)

-- Returns a table containing the fillType information (error on failure)

fillType = xAssert(Utils.getNoNil(tonumber(fillType), fillType))
	return xAssert(Utils.getNoNil(ucMod.fillTypesByIndex[fillType], ucMod.fillTypesByName[fillType]))
end

function UnitConvert:bulkDensityToMassPerLiter(bulkDensity)
-- This function converts a bulkDensity value to a massPerLiter value
	-- [bulkDensity]: Bulk density value to convert (kg/m3)

-- Returns the massPerLiter value (mt/l)

	if not getIsNumber(bulkDensity) then
		raiseError()
		return
	end

	if bulkDensity > ucMod.FILLTYPE_BULKDENSITY_MAX then bulkDensity = ucMod.FILLTYPE_BULKDENSITY_MAX end
	if bulkDensity < ucMod.FILLTYPE_BULKDENSITY_MIN then bulkDensity = ucMod.FILLTYPE_BULKDENSITY_MIN end

	return bulkDensity / 1000000
end

function UnitConvert:massPerLiterToBulkDensity(massPerLiter)
-- This function converts a massPerLiter value to a bulkDensity value
	-- [massPerLiter]: Mass per liter value to convert (mt/l)

-- Returns the bulkDensity value (kg/m3)

	if not getIsNumber(massPerLiter) then
		raiseError()
		return
	end

	local bulkDensity = massPerLiter * 1000000
	bulkDensity = math.floor((bulkDensity / ucMod.FILLTYPE_BULKDENSITY_STEP)) * ucMod.FILLTYPE_BULKDENSITY_STEP
	bulkDensity = Utils.clamp(bulkDensity, ucMod.FILLTYPE_BULKDENSITY_MIN, ucMod.FILLTYPE_BULKDENSITY_MAX)
	return bulkDensity
end

function UnitConvert:getHeaviestFillType(fillTypes)
-- This function finds the heaviest fillType among the list of fillTypes specified
	-- [fillTypes]:	List of fillTypes to check (by index, list, or table)

-- Returns the index of the heaviest fillType

	if not fillTypes then
		raiseError()
		return
	end

	if getIsNumber(fillTypes) then
		fillTypes = {fillTypes}
	else
		if not getIsTable(fillTypes) then
			fillTypes = splitString(" ", fillTypes)
		end
	end

	local maxMass = 0
	local retFillType = FillUtil.FILLTYPE_UNKNOWN

	for _, fillType in pairs(fillTypes) do
		if ucMod:getIsFillTypeValid(fillType) then
			fillType = ucMod:getFillType(fillType)
			local fillTypeMass = Utils.getNoNil(fillType.bulkDensity, ucMod:massPerLiterToBulkDensity(fillType.massPerLiter))
			if fillTypeMass >= maxMass then
				maxMass = fillTypeMass
				retFillType = fillType.index
			end
		end
	end

	if retFillType and retFillType ~= FillUtil.FILLTYPE_UNKNOWN then
		return retFillType
	end

	raiseError()
end

function UnitConvert:getFillTypeMass(fillType)
-- This function retrieves the display mass of the specified fillType
	-- [fillType]: FillType to retrieve

-- Returns the display massPerLiter value of the specified fillType (mt/l)

	if not ucMod:getIsFillTypeValid(fillType) then
		raiseError()
		return
	end

	fillType = ucMod:getFillType(fillType)

	local bulkDensity = fillType.bulkDensity
	if bulkDensity then
		return ucMod:bulkDensityToMassPerLiter(bulkDensity)
	end

	return fillType.massPerLiter
end

function UnitConvert:getVehicleFillMass(vehicle)
-- This function retrieves the original (internal) and new (display) mass of the specified vehicle
	-- [vehicle]: Vehicle used to calculate fillType mass

-- Returns
	--	[1]: The original fillType mass of the vehicle
	--	[2]: The display fillType mass of the vehicle

	if vehicle == nil or not getIsTable(vehicle) then
		raiseError()
		return
	end

	local massOld = 0
	local massNew = 0
    if vehicle.fillUnits then
        for _, vehicleFillUnit in pairs(vehicle.fillUnits) do
			local fillType = vehicleFillUnit.currentFillType
			if ucMod:getIsFillTypeValid(fillType) then
				local fillLevel = Utils.getNoNil(vehicleFillUnit.fillLevel, 0)
				fillType = ucMod:getFillType(fillType)

				local massPerLiterOld = fillType.massPerLiter
				local massPerLiterNew = ucMod:getFillTypeMass(fillType.index)

				massOld = massOld + (fillLevel * massPerLiterOld)
				massNew = massNew + (fillLevel * massPerLiterNew)
            end
        end
    end
    return massOld, massNew
end

function UnitConvert:convertValue(value, unit, fillType)
-- This function converts a value to the specified unit of measurement
	-- [value]:    Value to convert
	-- [unit]:     Unit to convert value to (by name or index)
	-- [fillType]: FillType used for fillType weight conversions (this value must be "false" for non-fillType conversions)

-- Returns the value converted to the specified unit

	if not getIsNumber(value) or (not getIsNumber(unit) and not getIsString(unit)) or fillType == nil then
		raiseError()
		return
	end

	unit = ucMod:getUnit(unit)
	if fillType and unit.unitType == ucMod.UNIT_TYPE_WEIGHT then
		value = value * ucMod:getFillTypeMass(fillType)
	end
	value = value * unit.conversionFactor

	return value
end

-- [THIRD PARTY ADDONS] *******************************************************************************************************************************************************************************

function UnitConvert:loadThirdPartyAddOns()
	for idx, addOn in ipairs(ucMod.addOns) do
		addOn.index = idx

		local source = addOn.source
		local name = Utils.getNoNil(addOn.name, source)

		for modName, isValid in pairs(g_modIsLoaded) do
			if isValid then
				local modDesc = ModsUtil.modNameToMod[modName]
				if modDesc and modDesc.version then
					local mod = getfenv(0)[modName]
					if mod and mod[source] then
						if modDesc.version == addOn.version then
							addOn.target = mod[source]
							if addOn:initialize() then
								debugMsg(-1, "UC_info_addOnLoaded", name, modDesc.version)
							else
								debugMsg(-1, "UC_error_addOnLoadFailed", name, modDesc.version)
							end
						else
							debugMsg(-1, "UC_warning_addOnVersionInvalid", name, modDesc.version, addOn.version)
						end
					end
				end
			end
		end
	end
end

-- [CONFIGURATION FILES] ******************************************************************************************************************************************************************************

function UnitConvert:getModI18N(modName)
-- This function retrieves the l10n information from external mods and adds it to UnitConvert's l10n (if the name begins with "UC_")
-- It does NOT overwrite any of UnitConvert's default l10n entries, this is for adding new texts
	-- [modName]: Internal name of external mod (global name (i.e. file name), not table name)

	modName = getfenv(0)[modName]
	if xAssert(modName and modName.g_i18n) then
		for name in pairs(modName.g_i18n.texts) do
			if string.find(name, "^UC_") then
				if not g_i18n:hasText(name) then
					g_i18n:setText(name, modName.g_i18n:getText(name))
				end
			end
		end
	end
end

function UnitConvert:addConfigXML(fileName, allowNew, allowOverwrite)
-- This function adds a configuration file to the configuration files table
	-- [fileName]:		 FileName to add
	-- [allowNew]:		 If true, allows this configuration file to add new entries
	-- [allowOverwrite]: If true, allows this configuration file to overwrite existing entries

-- Returns success (or nil on failure)

	if not getIsString(fileName)
	or (allowNew and not getIsBoolean(allowNew))
	or (allowOverwrite and not getIsBoolean(allowOverwrite))
	then
		raiseError()
		return
	end

	if fileExists(fileName) then
		table.insert(ucMod.configXMLs, {fileName = fileName, allowNew = allowNew, allowOverwrite = allowOverwrite})
		return true
	end
end

function UnitConvert:loadConfig(loadDefaults)
-- This function loads UnitConvert's configuration information from file(s)
	-- [loadDefaults]: If true, skips the user configuration file (for loading default values)

	if loadDefaults and not getIsBoolean(loadDefaults) then
		raiseError()
		return
	end

	local configFileName = "unitConvert"
	ucMod.configXMLs = {}

	-- Add default config
	xAssert(ucMod:addConfigXML(ucMod.modDir..configFileName..".xml", true, true))

	-- Add mod map (if applicable)
	-- Mod maps are allowed to add or overwrite anything
	local mapName = g_modSelectionScreen.mapModName
	if mapName then
		local mapFile = g_modNameToDirectory[mapName]..configFileName..".xml"
		if ucMod:addConfigXML(mapFile, true, true) then
			ucMod:getModI18N(mapName)
		end
	end

	-- Add additional loaded mods (if any)
	-- External mods are allowed to add, but not overwrite existing data
	for modName, isValid in pairs(g_modIsLoaded) do
		if isValid then
			local modFile = g_modNameToDirectory[modName]..configFileName..".xml"
			if modName ~= mapName and modName ~= g_i18n.modName and ucMod:addConfigXML(modFile, true, false) then
				ucMod:getModI18N(modName)
			end
		end
	end

	-- Add user configuration file last
	if not loadDefaults then
		local profilePath = getUserProfileAppPath()

		ucMod.saveFile = profilePath..configFileName.."_BETA.xml"
		ucMod:addConfigXML(ucMod.saveFile, true, true)

		if not ucMod.isBeta then
			ucMod.saveFile = profilePath..configFileName..".xml"
			ucMod:addConfigXML(ucMod.saveFile, true, true)
		end
	end

	if getTableLength(ucMod.configXMLs) == 0 then
		raiseError()
		return
	end

	-- Initialize variables
	ucMod.unitsByIndex = {}
	ucMod.unitsByName = {}

	ucMod.fillTypeGroupsByIndex = {}
	ucMod.fillTypeGroupsByName = {}

	local userFillTypeGroups = nil

	ucMod.fillTypesByIndex = {}
	ucMod.fillTypesByName = {}

	-- Load configuration data
	for configIdx, configData in ipairs(ucMod.configXMLs) do
		local fileName = configData.fileName
		local allowNew = configData.allowNew
		local allowOverwrite = configData.allowOverwrite

		local baseKey = "UnitConvert"
		local configXML = loadXMLFile(baseKey, fileName)
		if hasXMLProperty(configXML, baseKey) then

			local skipConfig = false

			local configVersion = Utils.getNoNil(getXMLInt(configXML, baseKey.."#version"), 1)
			if configIdx == 1 then
				ucMod.configVersion = configVersion
			else
				if configVersion < ucMod.configVersion then
					debugMsg(-1, "UC_warning_configIsOld", fileName)
					skipConfig = true
				end
			end

			if not skipConfig then
				if fileName == ucMod.saveFile then
					local yieldDisplay = Utils.getNoNil(getXMLInt(configXML, baseKey.."#yieldDisplay"), ucMod.yieldDisplay)
					if yieldDisplay < 1 or yieldDisplay > getTableLength(ucMod.yieldDisplayTypes) then
						yieldDisplay = ucMod.yieldDisplay
					end
					ucMod.yieldDisplay = yieldDisplay
				end

				-- UNITS

				local idx = 0
				while true do
					local unitKey = string.format("%s.unit(%d)", baseKey, idx)
					if not hasXMLProperty(configXML, unitKey) then
						break
					end

					local unitName = getXMLString(configXML, unitKey.."#name")
					if not unitName then
						debugMsg(-1, "UC_warning_unitInvalid", fileName)
					else
						local unitType				= getXMLString(configXML, unitKey.."#type")
						local unitPrecision			= getXMLInt(configXML, unitKey.."#precision")
						local unitConversionFactor	= getXMLFloat(configXML, unitKey.."#conversionFactor")
						local unitNameI18N			= getText("UC_unit_"..unitName)
						local unitNameI18NShort		= getText("UC_unit_"..unitName.."Short")

						local unit = ucMod.unitsByName[unitName]

						if unitType then
							unitType = ucMod.unitTypesByName[unitType]
							if not unitType then
								debugMsg(-1, "UC_warning_unitInvalidType", fileName, unitName)
							else
								unitType = unitType.index
							end
						end

						if unitPrecision then
							if unitPrecision < ucMod.UNIT_PRECISION_MIN or unitPrecision > ucMod.UNIT_PRECISION_MAX then
								debugMsg(-1, "UC_warning_unitInvalidPrecision", fileName, unitName)
								unitPrecision = nil
							elseif unitType == ucMod.UNIT_TYPE_PIECE then
								unitPrecision = nil
							end
						end

						if unitConversionFactor then
							if unitConversionFactor <= 0 then
								debugMsg(-1, "UC_warning_unitInvalidConversionFactor", fileName, unitName)
								unitConversionFactor = nil
							elseif unitType == ucMod.UNIT_TYPE_PIECE then
								unitConversionFactor = nil
							end
						end

						if unit then
							if allowOverwrite then
								unit.precision			= Utils.getNoNil(unitPrecision, unit.precision)
								unit.conversionFactor	= Utils.getNoNil(unitConversionFactor, unit.conversionFactor)
							end
						else
							if allowNew then
								local unitIndex = getTableLength(ucMod.unitsByIndex) + 1
								unit = {
									name				= unitName,
									index				= unitIndex,
									unitType			= Utils.getNoNil(unitType, ucMod.UNIT_TYPE_PIECE),
									precision			= Utils.getNoNil(unitPrecision, 0),
									conversionFactor	= Utils.getNoNil(unitConversionFactor, 1),
									nameI18N			= unitNameI18N,
									nameI18NShort		= unitNameI18NShort
								}

								table.insert(ucMod.unitsByIndex, unit)
								ucMod.unitsByName[unit.name] = unit
								ucMod["UNIT_"..string.upper(unit.name)] = unit.index
								debugMsg(1, "UC_info_unitRegistered", unit.name)
							end
						end
					end
					idx = idx + 1
				end

				-- DEFAULT UNITS

				if configIdx == 1 then -- Base config
					for _, unitType in pairs(ucMod.unitTypesByIndex) do
						local defaultUnit = ucMod:getDefaultUnit(unitType.index, true)
						unitType.defaultUnit = defaultUnit.index
					end
				end

				if allowOverwrite then
					local idx = 0
					while true do
						local unitTypeKey = string.format("%s.unitType(%d)", baseKey, idx)
						if not hasXMLProperty(configXML, unitTypeKey) then
							break
						end

						local unitTypeName = getXMLString(configXML, unitTypeKey.."#name")
						local unitType = ucMod.unitTypesByName[unitTypeName]
						if not unitType then
							debugMsg(-1, "UC_warning_unitTypeInvalid", fileName)
						else
							local unitTypeDefaultUnit = getXMLString(configXML, unitTypeKey.."#defaultUnit")
							if unitTypeDefaultUnit then
								local unit = ucMod.unitsByName[unitTypeDefaultUnit]
								if not unit or unit.unitType ~= unitType.index then
									debugMsg(-1, "UC_warning_unitTypeInvalidDefaultUnit", fileName, unitType.name, unit.name)
								else
									unitType.defaultUnit = unit.index
								end
							end
						end
						idx = idx + 1
					end
				end

				-- FILLTYPE GROUPS

				local idx = 0
				while true do
					local fillTypeGroupKey = string.format("%s.fillTypeGroup(%d)", baseKey, idx)
					if not hasXMLProperty(configXML, fillTypeGroupKey) then
						break
					end

					local fillTypeGroupName = getXMLString(configXML, fillTypeGroupKey.."#name")
					if not fillTypeGroupName then
						debugMsg(-1, "UC_warning_fillTypeGroupInvalid", fileName)
					else
						local fillTypeGroupPriority		= getXMLInt(configXML, fillTypeGroupKey.."#priority")
						local fillTypeGroupUnitTypes	= getXMLString(configXML, fillTypeGroupKey.."#unitTypes")
						local fillTypeGroupLockUnits	= getXMLBool(configXML, fillTypeGroupKey.."#lockUnits")
						local fillTypeGroupIsDefault	= getXMLBool(configXML, fillTypeGroupKey.."#isDefaultGroup")
						local fillTypeGroupUnits		= {}

						local fillTypeGroup = ucMod.fillTypeGroupsByName[fillTypeGroupName]

						if fillTypeGroupPriority then
							if fillTypeGroupPriority <= 0 then
								debugMsg(-1, "UC_warning_fillTypeGroupInvalidPriority", fileName, fillTypeGroupName)
								fillTypeGroupPriority = nil
							end
						end

						if fillTypeGroupUnitTypes then
							fillTypeGroupUnitTypes = splitString(" ", fillTypeGroupUnitTypes)
							for unitTypeIndex, unitTypeName in pairs(fillTypeGroupUnitTypes) do
								unitType = ucMod.unitTypesByName[unitTypeName]
								if not unitType or unitType.index == ucMod.UNIT_TYPE_AREA or unitType.index == ucMod.UNIT_TYPE_LENGTH then
									debugMsg(-1, "UC_warning_fillTypeGroupInvalidUnitType", fileName, fillTypeGroupName)
									fillTypeGroupUnitTypes[unitTypeIndex] = nil
								elseif unitType.index == ucMod.UNIT_TYPE_PIECE then -- Piece unit types cannot co-exist with other unit types
									fillTypeGroupUnitTypes = {}
									fillTypeGroupUnitTypes[unitType.index] = true
									break
								else
									fillTypeGroupUnitTypes[unitType.index] = true
								end
							end
						end

						if not fillTypeGroupUnitTypes or fillTypeGroupUnitTypes == {} then
							fillTypeGroupUnitTypes = ucMod:getDefaultUnitTypes()
						end

						for _, unitSet in ipairs(ucMod.unitSetsByIndex) do
							local unitSetKey = unitSet.name.."Unit"
							fillTypeGroupUnits[unitSet.index] = getXMLString(configXML, fillTypeGroupKey.."#"..unitSetKey)
							if fillTypeGroupUnits[unitSet.index] then
								local unit = ucMod.unitsByName[fillTypeGroupUnits[unitSet.index]]

								local allowedUnitTypes = fillTypeGroupUnitTypes
								if fillTypeGroup then
									allowedUnitTypes = fillTypeGroup.unitTypes
								end

								if not unit or not allowedUnitTypes[unit.unitType] then
									debugMsg(-1, "UC_warning_fillTypeGroupInvalidUnit", fileName, fillTypeGroupName, unitSetKey)
									fillTypeGroupUnits[unitSet.index] = nil
								else
									fillTypeGroupUnits[unitSet.index] = unit.index
								end
							end
						end

						if configIdx > 1 then
							fillTypeGroupLockUnits = nil

							if fileName ~= ucMod.saveFile then
								fillTypeGroupIsDefault = true
							end
						end

						if fillTypeGroup then
							if allowOverwrite then
								fillTypeGroup.priority = Utils.getNoNil(fillTypeGroupPriority, fillTypeGroup.priority)

								if not fillTypeGroup.lockUnits then
									for unitSetIdx in pairs(fillTypeGroup.units) do
										if fillTypeGroupUnits[unitSetIdx] then
											local unit = ucMod.unitsByIndex[fillTypeGroupUnits[unitSetIdx]]
											if fillTypeGroup.unitTypes[unit.unitType] then
												fillTypeGroup.units[unitSetIdx] = fillTypeGroupUnits[unitSetIdx]
											end
										end
									end
								end
							end
						else
							if allowNew then
								local fillTypeGroupIndex = getTableLength(ucMod.fillTypeGroupsByIndex) + 1
								fillTypeGroup = {
									name		= fillTypeGroupName,
									index		= fillTypeGroupIndex,
									priority	= Utils.getNoNil(fillTypeGroupPriority, fillTypeGroupIndex),
									unitTypes	= fillTypeGroupUnitTypes,
									lockUnits	= fillTypeGroupLockUnits,
									isDefault	= fillTypeGroupIsDefault,
									units 		= {}
								}

								for _, unitSet in pairs(ucMod.unitSetsByIndex) do
									if fillTypeGroupUnits[unitSet.index] then
										local unit = ucMod.unitsByIndex[fillTypeGroupUnits[unitSet.index]]
										if not fillTypeGroup.unitTypes[unit.unitType] then
											fillTypeGroupUnits[unitSet.index] = nil
										end
									end
									local defaultUnit = ucMod:getDefaultUnit(fillTypeGroup.unitTypes, true)
									fillTypeGroup.units[unitSet.index] = Utils.getNoNil(fillTypeGroupUnits[unitSet.index], defaultUnit.index)
								end

								table.insert(ucMod.fillTypeGroupsByIndex, fillTypeGroup)
								ucMod.fillTypeGroupsByName[fillTypeGroup.name] = fillTypeGroup
								ucMod["FILLTYPE_GROUP_"..string.upper(fillTypeGroup.name)] = fillTypeGroup.index
								debugMsg(1, "UC_info_fillTypeGroupRegistered", fillTypeGroup.name)
							end
						end
						if fileName == ucMod.saveFile then
							userFillTypeGroups = Utils.getNoNil(userFillTypeGroups, {})
							userFillTypeGroups[fillTypeGroup.name] = true
						end
					end
					idx = idx + 1
				end

				-- FILLTYPES

				local idx = 0
				while true do
					local fillTypeKey = string.format("%s.fillType(%d)", baseKey, idx)
					if not hasXMLProperty(configXML, fillTypeKey) then
						break
					end

					local fillTypeName = getXMLString(configXML, fillTypeKey.."#name")
					if not fillTypeName then
						debugMsg(-1, "UC_warning_fillTypeInvalid", fileName)
					elseif not FillUtil.fillTypeNameToInt[fillTypeName] then
						debugMsg(10, "UC_warning_fillTypeMissing", fileName, fillTypeName)
					else
						local fillTypeIndex				= FillUtil.fillTypeNameToInt[fillTypeName]
						local fillTypeGroup				= getXMLString(configXML, fillTypeKey.."#group")
						local fillTypeShowOnMenu		= getXMLBool(configXML, fillTypeKey.."#showOnMenu")
						local fillTypeShowOnPriceTable	= getXMLBool(configXML, fillTypeKey.."#showOnPriceTable")
						local fillTypeBulkDensity		= getXMLInt(configXML, fillTypeKey.."#bulkDensity")
						local fillTypeMassPerLiter		= FillUtil.fillTypeIndexToDesc[fillTypeIndex].massPerLiter

						local fillType = ucMod.fillTypesByName[fillTypeName]

						if fillTypeGroup then
							fillTypeGroup = ucMod.fillTypeGroupsByName[fillTypeGroup]
							if not fillTypeGroup then
								debugMsg(-1, "UC_warning_fillTypeInvalidGroup", fileName, fillTypeName)
							else
								fillTypeGroup = fillTypeGroup.index
							end
						end

						if fillTypeBulkDensity then
							if fillTypeBulkDensity < ucMod.FILLTYPE_BULKDENSITY_MIN or fillTypeBulkDensity > ucMod.FILLTYPE_BULKDENSITY_MAX then
								debugMsg(-1, "UC_warning_fillTypeInvalidBulkDensity", fileName, fillTypeName, fillTypeBulkDensity)
								fillTypeBulkDensity = nil
							end
						end

						if configIdx > 1 then
							fillTypeShowOnMenu = nil
						end

						if fillType then
							if allowOverwrite then
								fillType.group				= Utils.getNoNil(fillTypeGroup, fillType.group)
								fillType.bulkDensity		= Utils.getNoNil(fillTypeBulkDensity, fillType.bulkDensity)
								fillType.showOnPriceTable	= Utils.getNoNil(fillTypeShowOnPriceTable, fillType.showOnPriceTable)
							end
						else
							fillType = {
								name				= fillTypeName,
								index				= fillTypeIndex,
								group				= Utils.getNoNil(fillTypeGroup, ucMod:getDefaultFillTypeGroup(fillTypeIndex)),
								showOnMenu			= Utils.getNoNil(fillTypeShowOnMenu, true),
								showOnPriceTable	= Utils.getNoNil(fillTypeShowOnPriceTable, FillUtil.fillTypeIndexToDesc[fillTypeIndex].showOnPriceTable),
								bulkDensity			= fillTypeBulkDensity
							}

							ucMod.fillTypesByIndex[fillType.index] = fillType
							ucMod.fillTypesByName[fillType.name] = fillType
							debugMsg(1, "UC_info_fillTypeRegistered", fillType.name)
						end
					end
					idx = idx + 1
				end
			end
		end
		delete(configXML)
	end

	-- Finalize unit registration
	ucMod.UNIT_UNKNOWN = 0
	ucMod.NUM_UNITS = getTableLength(ucMod.unitsByIndex)

	-- Finalize fillType group registration
	ucMod.FILLTYPE_GROUP_UNKNOWN = 0

	-- Remove user deleted default fillType groups
	if userFillTypeGroups then
		local fillTypeGroupsToRemove = {}
		for _, fillTypeGroup in pairs(ucMod.fillTypeGroupsByIndex) do
			if not fillTypeGroup.isDefault and not userFillTypeGroups[fillTypeGroup.name] then
				table.insert(fillTypeGroupsToRemove, fillTypeGroup.name)
			end
		end

		for _, name in pairs(fillTypeGroupsToRemove) do
			ucMod:removeFillTypeGroup(name)
		end
	end

	ucMod.NUM_FILLTYPE_GROUPS = getTableLength(ucMod.fillTypeGroupsByIndex)
	ucMod.fillTypeGroupsByPriority = indexTableByKey(ucMod.fillTypeGroupsByIndex, "priority")

	-- Finalize fill type registration
	for _, fillType in pairs(FillUtil.fillTypeIndexToDesc) do
		if fillType.index ~= FillUtil.FILLTYPE_UNKNOWN then
			local ucFillType = ucMod.fillTypesByIndex[fillType.index]
			if not ucFillType then
				ucFillType = {
					name				= fillType.name,
					index				= fillType.index,
					group				= ucMod:getDefaultFillTypeGroup(fillType.index),
					showOnMenu			= true,
					showOnPriceTable	= fillType.showOnPriceTable
				}

				ucMod.fillTypesByIndex[ucFillType.index] = ucFillType
				ucMod.fillTypesByName[ucFillType.name] = ucFillType
			end

			local massPerLiter = Utils.getNoNil(fillType.massPerLiter, 0)
			local massPerLiterMin = ucMod:bulkDensityToMassPerLiter(ucMod.FILLTYPE_BULKDENSITY_MIN)

			if massPerLiter <= massPerLiterMin then
				if ucFillType.index == FillUtil.FILLTYPE_POWERFOOD then
					massPerLiter = FillUtil.fillTypeIndexToDesc[FillUtil.FILLTYPE_FORAGE].massPerLiter
				else
					massPerLiter = massPerLiterMin
				end
			end

			ucFillType.nameI18N = fillType.nameI18N
			ucFillType.massPerLiter = massPerLiter

			if ucFillType.bulkDensity then
				local massPerLiter = ucMod:bulkDensityToMassPerLiter(ucFillType.bulkDensity)
				if massPerLiter == ucFillType.massPerLiter then
					ucFillType.bulkDensity = nil
				else
					ucFillType.bulkDensity = ucMod:massPerLiterToBulkDensity(massPerLiter) -- Do this to align value to allowed values
				end
			end

			fillType.showOnPriceTable = ucFillType.showOnPriceTable
		end
	end
end

function UnitConvert:saveConfig()
-- This function saves UnitConvert's configuration information to file

	local baseKey = "UnitConvert"
	local saveXML = createXMLFile(baseKey, ucMod.saveFile, baseKey)

	if xAssert(saveXML) then
		setXMLInt(saveXML, baseKey.."#version", ucMod.configVersion)
		setXMLInt(saveXML, baseKey.."#yieldDisplay", ucMod.yieldDisplay)

		local idx = 0
		for _, unit in ipairs(ucMod.unitsByIndex) do
			local unitKey = string.format("%s.unit(%d)", baseKey, idx)
			setXMLString(saveXML, unitKey.."#name", unit.name)
			setXMLInt(saveXML, unitKey.."#precision", unit.precision)
			idx = idx + 1
		end

		local idx = 0
		for _, unitType in ipairs(ucMod.unitTypesByIndex) do
			local unitTypeKey = string.format("%s.unitType(%d)", baseKey, idx)
			local defaultUnit = ucMod:getUnit(unitType.defaultUnit)
			setXMLString(saveXML, unitTypeKey.."#name", unitType.name)
			setXMLString(saveXML, unitTypeKey.."#defaultUnit", defaultUnit.name)
			idx = idx + 1
		end

		local idx = 0
		for _, fillTypeGroup in ipairs(ucMod.fillTypeGroupsByIndex) do
			local fillTypeGroupKey = string.format("%s.fillTypeGroup(%d)", baseKey, idx)
			setXMLString(saveXML, fillTypeGroupKey.."#name", fillTypeGroup.name)
			setXMLInt(saveXML, fillTypeGroupKey.."#priority", fillTypeGroup.priority)
			for _, unitSet in ipairs(ucMod.unitSetsByIndex) do
				if fillTypeGroup.units[unitSet.index] then
					local unit = ucMod:getUnit(fillTypeGroup.units[unitSet.index])
					setXMLString(saveXML, string.format("%s#%sUnit", fillTypeGroupKey, unitSet.name), unit.name)
				end
			end
			idx = idx + 1
		end

		local idx = 0
		for _, fillType in ipairs(ucMod.fillTypesByIndex) do
			if fillType.showOnMenu then
				local fillTypeKey = string.format("%s.fillType(%d)", baseKey, idx)
				local fillTypeGroup = ucMod:getFillTypeGroup(fillType.group)
				setXMLString(saveXML, fillTypeKey.."#name", fillType.name)
				setXMLString(saveXML, fillTypeKey.."#group", fillTypeGroup.name)

				if fillType.bulkDensity then
					setXMLInt(saveXML, fillTypeKey.."#bulkDensity", fillType.bulkDensity)
				end

				idx = idx + 1
			end
		end
		saveXMLFile(saveXML)
		delete(saveXML)
	end
end

-- [MAIN PROGRAM] *************************************************************************************************************************************************************************************

function UnitConvert:loadMap(name)
	-- Unit precision constants
	ucMod.UNIT_PRECISION_MIN	= 0
	ucMod.UNIT_PRECISION_MAX	= 4
	ucMod.UNIT_PRECISION_STEP	= 1

	-- FillType bulk density constants
	ucMod.FILLTYPE_BULKDENSITY_MIN	= 5
	ucMod.FILLTYPE_BULKDENSITY_MAX	= 5000
	ucMod.FILLTYPE_BULKDENSITY_STEP	= 5

	-- Yield display
	ucMod.yieldDisplayTypes = {}
	table.insert(ucMod.yieldDisplayTypes, "off")
	table.insert(ucMod.yieldDisplayTypes, "vehicle")
	table.insert(ucMod.yieldDisplayTypes, "price")
	for k, v in ipairs(ucMod.yieldDisplayTypes) do
		ucMod["YIELD_DISPLAY_"..string.upper(v)] = k
	end
	ucMod.yieldDisplay = ucMod.YIELD_DISPLAY_VEHICLE

	-- Color constants
	ucMod.colors = {}
	ucMod.colors.white				= {1, 1, 1, 1}
	ucMod.colors.red				= {0.8069, 0.0097, 0.0097, 1}
	ucMod.colors.green				= {0.2122, 0.5271, 0.0307, 1}
	ucMod.colors.green2 			= {0.2705, 0.6514, 0.0802, 1}
	ucMod.colors.textUnitColor		= {0.0865, 0.0865, 0.0865, 1}
	ucMod.colors.textUnitColor2		= {0.3365, 0.3365, 0.3365, 1}

	-- Unit types
	ucMod.unitTypesByIndex = {}
	table.insert(ucMod.unitTypesByIndex, {name = "volume",	priority = 4})
	table.insert(ucMod.unitTypesByIndex, {name = "weight",	priority = 5})
	table.insert(ucMod.unitTypesByIndex, {name = "length",	priority = 2})
	table.insert(ucMod.unitTypesByIndex, {name = "area",	priority = 1})
	table.insert(ucMod.unitTypesByIndex, {name = "piece",	priority = 3})

	ucMod.unitTypesByName = {}
	ucMod.unitTypesByPriority = {}
	for unitTypeIdx, unitType in pairs(ucMod.unitTypesByIndex) do
		unitType.index = unitTypeIdx
		ucMod.unitTypesByName[unitType.name] = unitType
		ucMod.unitTypesByPriority[unitType.priority] = unitType
		ucMod["UNIT_TYPE_"..string.upper(unitType.name)] = unitType.index
	end
	ucMod.NUM_UNIT_TYPES = getTableLength(ucMod.unitTypesByIndex)

	-- Unit sets
	ucMod.unitSetsByIndex = {}
	table.insert(ucMod.unitSetsByIndex, {name = "vehicle"})
	table.insert(ucMod.unitSetsByIndex, {name = "storage"})
	table.insert(ucMod.unitSetsByIndex, {name = "animal"})
	table.insert(ucMod.unitSetsByIndex, {name = "price"})

	ucMod.unitSetsByName = {}
	for unitSetIdx, unitSet in pairs(ucMod.unitSetsByIndex) do
		unitSet.index = unitSetIdx
		ucMod.unitSetsByName[unitSet.name] = unitSet
		ucMod["UNIT_SET_"..string.upper(unitSet.name)] = unitSet.index
	end
	ucMod.NUM_UNIT_SETS = getTableLength(ucMod.unitSetsByIndex)

	-- Seasons (or any mod that changes AnimalUtil data) compatibility
	ucMod.animalsByIndex = {}
	ucMod.animalsByName = {}
	for _, animal in pairs(AnimalUtil.animalIndexToDesc) do
		ucMod.animalsByIndex[animal.index] = {}

		ucMod.animalsByIndex[animal.index].name = animal.name
		ucMod.animalsByIndex[animal.index].index = animal.index

		ucMod.animalsByIndex[animal.index].hasWater = (animal.waterPerDay > 0)
		ucMod.animalsByIndex[animal.index].hasStraw = (animal.strawPerDay > 0)
		ucMod.animalsByIndex[animal.index].hasPallet = (animal.palletFillLevelPerDay > 0)
		ucMod.animalsByIndex[animal.index].hasManure = (animal.manurePerDay > 0)
		ucMod.animalsByIndex[animal.index].hasLiquidManure = (animal.liquidManurePerDay > 0)
		ucMod.animalsByIndex[animal.index].hasMilk = (animal.milkPerDay > 0)

		ucMod.animalsByName[animal.name] = ucMod.animalsByIndex[animal.index]
	end
end

function UnitConvert:postLoadMapFinished(superFunc, ...)
	local ret = {superFunc(self, ...)}

--[[ Do most things here to prevent being overwritten by other mods --]]

	-- Load Configuration(s)
	ucMod:loadConfig()

	-- Make UnitConvert Texts Global
	for key, text in pairs(g_i18n.texts) do
		getfenv(0)["g_i18n"]:setText(key, text)
	end

	-- Load Configuration Menu
    g_gui:loadProfiles(ucMod.modDir.."gui/ucProfiles.xml")

	ucMod.cfgMenu = UCMenu:new()
	g_gui:loadGui(ucMod.modDir.."gui/ucMenu.xml", "UCMenu", ucMod.cfgMenu)

    FocusManager:setGui("MPLoadingScreen")

	-- Load Additional Mods
	ucMod:loadThirdPartyAddOns()

	-- Function Hooks
		-- InGameMenu - Prices Page
		g_inGameMenu.setupPriceTable	= Utils.overwrittenFunction(g_inGameMenu.setupPriceTable, ucMod.inGameMenuSetupPriceTable)
		g_inGameMenu.updatePriceTable	= Utils.overwrittenFunction(g_inGameMenu.updatePriceTable, ucMod.inGameMenuUpdatePriceTable)

		-- InGameMenu - Statistics Page
		g_inGameMenu.updateStatisticTable = Utils.overwrittenFunction(g_inGameMenu.updateStatisticTable, ucMod.inGameMenuUpdateStatisticTable)

		-- InGameMenu - Animals Page
		g_inGameMenu.getAnimalData = Utils.overwrittenFunction(g_inGameMenu.getAnimalData, ucMod.inGameMenuGetAnimalData)

		-- ShopScreen / ShopScreenConfig
		g_shopScreen.updateAttributes = Utils.overwrittenFunction(g_shopScreen.updateAttributes, ucMod.shopScreenUpdateAttributes)
		g_shopConfigScreen.updateData = Utils.overwrittenFunction(g_shopConfigScreen.updateData, ucMod.shopConfigScreenUpdateData)

		-- SiloDialog
		g_gui.showSiloDialog = Utils.overwrittenFunction(g_gui.showSiloDialog, ucMod.guiShowSiloDialog)

		-- FieldJobMissionScreen
		g_fieldJobMissionScreen.setData = Utils.overwrittenFunction(g_fieldJobMissionScreen.setData, ucMod.fieldJobMissionScreenSetData)

		-- BGA
		Bga.setFillLevel = Utils.overwrittenFunction(Bga.setFillLevel, ucMod.bgaSetFillLevel)

		-- BunkerSilo
		BunkerSilo.update = Utils.overwrittenFunction(BunkerSilo.update, ucMod.bunkerSiloUpdate)

		-- LiquidManureFillTrigger
		LiquidManureFillTrigger.update = Utils.overwrittenFunction(LiquidManureFillTrigger.update, ucMod.liquidManureFillTriggerUpdate)

		-- WeighStation
		WeighStation.updateWeight = Utils.overwrittenFunction(WeighStation.updateWeight, ucMod.weighStationUpdateWeight)

		-- VehicleHud
		g_currentMission.drawLevelBar = Utils.overwrittenFunction(g_currentMission.drawLevelBar, ucMod.currentMissionDrawLevelBar)
		VehicleHudUtils.setHudValue = Utils.overwrittenFunction(VehicleHudUtils.setHudValue, ucMod.vehicleHudUtilsSetHudValue)

	-- FOR TESTING PURPOSES
	if DebugTools then
		--DebugTools:searchTable(ucMod, 2)
	end

	return unpack(ret)
end
BaseMission.loadMapFinished = Utils.overwrittenFunction(BaseMission.loadMapFinished, ucMod.postLoadMapFinished)

function UnitConvert:update(dt)
	g_currentMission:addHelpButtonText(getText("UC_help_openMenu"), InputBinding.UC_OPEN_MENU, nil, GS_PRIO_NORMAL)
	if InputBinding.hasEvent(InputBinding.UC_OPEN_MENU) then
		g_gui:showGui("UCMenu")
	end
end

function UnitConvert:draw() end
function UnitConvert:deleteMap() end
function UnitConvert:mouseEvent(posX, posY, isDown, isUp, button) end
function UnitConvert:keyEvent(unicode, sym, modifier, isDown) end

-- [INGAMEMENU OVERRIDES - PRICES PAGE] ***************************************************************************************************************************************************************

function UnitConvert:inGameMenuSetupPriceTable(superFunc, ...)
	if not self.unitConvert then self.unitConvert = {} end

	if not self.unitConvert.priceTableInitialized then
		self.fillTypeUnits = {}
		self.fillTypeNames = {}
		for i = 1, self.MAX_NUM_FILLTYPES do
			self.fillTypeUnits[i] = self.fillLevels[i]:clone(self.fillLevels[i].parent)
			self.fillTypeUnits[i]:setTextSize(getCorrectTextSize(self.fillLevels[i].defaultTextSize * 0.65))
			self.fillTypeUnits[i].textUpperCase = true
			self.fillTypeUnits[i].textBold = true

			local fillTypeUnitPosY = self.fillTypeUnits[i].parent.size[2] - self.fillTypeUnits[i]:getTextHeight() - 0.002
			self.fillTypeUnits[i]:setPosition(0, fillTypeUnitPosY)
			self.fillTypeUnits[i]:setTextColor(unpack(ucMod.colors.red))

			self.fillTypeNames[i] = self.fillLevels[i]:clone(self.fillLevels[i].parent)
			self.fillTypeNames[i]:setTextSize(getCorrectTextSize(self.fillLevels[i].defaultTextSize) * 0.65)
			self.fillTypeNames[i].textUpperCase = true
			self.fillTypeNames[i].textBold = true

			local fillTypeNamePosY = self.fillTypeNames[i].parent.size[2] + 0.002
			self.fillTypeNames[i]:setPosition(0, fillTypeNamePosY)
			self.fillTypeNames[i]:setTextColor(unpack(ucMod.colors.green2))

			self.fillLevels[i]:setPosition(0, 0.006)

			self.fillIcons[i].position[2] = self.fillIcons[i].position[2] + 0.005
			self.fillIcons[i]:setPosition(unpack(self.fillIcons[i].position))
		end

		for idx, page in pairs(self.pagingElement.pageMapping) do
			if page.id == InGameMenu.PAGE_PRICES then
				page.title = getText("UC_inGameMenu_prices")
				self.pageSelector.texts[idx] = page.title
				break
			end
		end

		local priceHeaderOffset = 0.01
		local priceHeaderBox = self.ingameMenuPriceHeaderBox
		local priceHeaderBoxFarm = priceHeaderBox.parent.elements[2]

		self.storageTitleElement = priceHeaderBoxFarm.elements[1].elements[1] -- TODO: find a easier way of finding these
		self.storageTitleElement:setText(getText("UC_inGameMenu_storageTotals"))

		for elementIdx, element in pairs(priceHeaderBox.elements) do
			element.size[2] = element.size[2] + priceHeaderOffset
			element:setSize(unpack(element.size))

			if elementIdx == 1 then
				element.elements[1]:updatePositionForOrigin("middleLeft")
				element.elements[1].position[2] = element.size[2] / 2
				element.elements[1]:setPosition(unpack(element.elements[1].position))
			end
		end

		priceHeaderBox.size[2] = priceHeaderBox.size[2] + priceHeaderOffset
		priceHeaderBox:setSize(unpack(priceHeaderBox.size))

		priceHeaderBoxFarm.position[2] = priceHeaderBoxFarm.position[2] - priceHeaderOffset
		priceHeaderBoxFarm:setPosition(unpack(priceHeaderBoxFarm.position))

		self.pricesList.position[2] = self.pricesList.position[2] - priceHeaderOffset
		self.pricesList:setPosition(unpack(self.pricesList.position))
		self.pricesList.visibleItems = self.pricesList.visibleItems - 1

		self.unitConvert.priceTableInitialized = true
	end

	local ret = {superFunc(self, ...)}
	return unpack(ret)
end

function UnitConvert:inGameMenuUpdatePriceTable(superFunc, doNotUpdateSlider, ...)
	local ret = {superFunc(self, doNotUpdateSlider, ...)}

	local function getTextScale(text)
		local ret = 1
		if string.len(text) >= 1 then
			ret = ret - (string.len(text) / 100)
		end
		return ret
	end

	for i = 1, InGameMenu.MAX_NUM_FILLTYPES do
		local priceFillType = self.priceFillTypes[self.startFillType + (i - 1)]
		if priceFillType then
			local fillType = priceFillType.index
			local fillLevel = 0
			for _, storage in pairs(g_currentMission.storages) do
				fillLevel = fillLevel + Utils.getNoNil(storage.fillLevels[fillType], 0)
			end

			local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_PRICE)
			fillLevel = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
			fillLevel = numToStr(fillLevel, fillUnit.precision, "ceil", true, false)

			local fillLevelTextSize = getCorrectTextSize(self.fillLevels[i].defaultTextSize) * 0.95
			self.fillLevels[i]:setTextSize(fillLevelTextSize * getTextScale(fillLevel))
			self.fillLevels[i]:setText(fillLevel)

			self.fillTypeUnits[i]:setText(fillUnit.nameI18N)
			self.fillTypeNames[i]:setText(FillUtil.fillTypeIndexToDesc[fillType].nameI18N)

			for _, tipTriggerElement in ipairs(self.tipTriggers) do
				local priceElement = tipTriggerElement.prices[i]
				local arrowElement = tipTriggerElement.arrows[i]
				local parentElement = priceElement.parent

				arrowElement:setPosition(-0.0038, 0)

				for _, tipTrigger in ipairs(tipTriggerElement.tipTriggers) do
					if tipTrigger.acceptedFillTypes[fillType] then
						local fillPrice = tipTrigger:getEffectiveFillTypePrice(fillType)
						local fillPriceFactor = ucMod:convertValue(1, fillUnit.index, fillType)
						fillPrice = fillPrice / fillPriceFactor
						fillPrice = numToStr(fillPrice, 2, "ceil", true, true)

						if string.len(fillPrice) > 6 then
						else
							priceElement:setTextSize(getCorrectTextSize(priceElement.defaultTextSize))
						end

						priceElement:setTextSize(getCorrectTextSize(priceElement.defaultTextSize) * getTextScale(fillPrice))
						priceElement:setText(string.format("%s %s", fillPrice, g_i18n:getCurrencySymbol(true)))
						break
					end
				end
				local priceElementX = 0 - arrowElement.size[1] - 0.0062
				local priceElementY = 0.0015 + ((parentElement.size[2] / 2) - (priceElement:getTextHeight() / 2))
				priceElement:setPosition(priceElementX, priceElementY)
			end
		end
	end

	return unpack(ret)
end

-- [INGAMEMENU OVERRIDES - STATISTICS PAGE] ***********************************************************************************************************************************************************

function UnitConvert:inGameMenuUpdateStatisticTable(superFunc, ...)
	local ret = {superFunc(self, ...)}

	for _, statistic in pairs(g_currentMission.missionStats.statisticData) do
		if statistic.valueSessionElem and statistic.valueTotalElem then
			statistic.valueTitleElem = statistic.valueTotalElem.parent.elements[1]

			local valueTitle = ""
			local valueSession = Utils.getNoNil(statistic.valueSession, "")
			local valueTotal = Utils.getNoNil(statistic.valueTotal, "")

			local fillType = FillUtil.FILLTYPE_UNKNOWN
			local statUnit = ucMod.UNIT_UNKNOWN
			local roundType = "floor"

			if string.find(statistic.name, getText("statistic_workedHectares")) then
				fillType = false
				statUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				valueTitle = string.format("%s [%s]", getText("UC_statistic_workedArea"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["workedHectares"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["workedHectares"].total
				end
			elseif string.find(statistic.name, getText("statistic_cultivatedHectares")) then
				fillType = false
				statUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				valueTitle = string.format("%s [%s]", getText("UC_statistic_cultivatedArea"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["cultivatedHectares"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["cultivatedHectares"].total
				end
			elseif string.find(statistic.name, getText("statistic_sownHectares")) then
				fillType = false
				statUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				valueTitle = string.format("%s [%s]", getText("UC_statistic_sownArea"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["sownHectares"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["sownHectares"].total
				end
			elseif string.find(statistic.name, getText("statistic_fertilizedHectares")) then
				fillType = false
				statUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				valueTitle = string.format("%s [%s]", getText("UC_statistic_fertilizedArea"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["fertilizedHectares"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["fertilizedHectares"].total
				end
			elseif string.find(statistic.name, getText("statistic_threshedHectares")) then
				fillType = false
				statUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				valueTitle = string.format("%s [%s]", getText("UC_statistic_threshedArea"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["threshedHectares"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["threshedHectares"].total
				end
			elseif string.find(statistic.name, getText("statistic_fuelUsage")) then
				fillType = FillUtil.FILLTYPE_FUEL
				statUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)
				valueTitle = string.format("%s [%s]", getText("statistic_fuelUsage"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["fuelUsage"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["fuelUsage"].total
				end
			elseif string.find(statistic.name, getText("statistic_seedUsage")) then
				fillType = FillUtil.FILLTYPE_SEEDS
				statUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)
				valueTitle = string.format("%s [%s]", getText("statistic_seedUsage"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["seedUsage"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["seedUsage"].total
				end
			elseif string.find(statistic.name, getText("statistic_sprayUsage")) then
				-- Not quite sure what to do with this one, as there are multiple types of fertilizer (both liquid and solid)
				-- For now, using "fertilizer" as the template, based on the idea that you mix solid fertilizer with water to make liquid...
				fillType = FillUtil.FILLTYPE_FERTILIZER
				statUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)
				valueTitle = string.format("%s [%s]", getText("statistic_sprayUsage"), statUnit.nameI18NShort)
				if valueSession ~= "" then
					valueSession = g_currentMission.missionStats.statistics["sprayUsage"].session
				end
				if valueTotal ~= "" then
					valueTotal = g_currentMission.missionStats.statistics["sprayUsage"].total
				end
			elseif string.find(statistic.name, getText("statistic_storageCapacity")) then
				statistic.valueTitleElem:setText(getText("UC_statistic_siloCapacity")) -- More accurate description
				if g_currentMission.farmSiloTrigger then
					local siloFillTypes = g_currentMission.farmSiloTrigger.fillTypes
					if getIsTable(siloFillTypes) and getTableLength(siloFillTypes) > 0 then
						fillType = ucMod:getHeaviestFillType(siloFillTypes)
						if fillType then
							statUnit = ucMod:getFillUnit(siloFillTypes, ucMod.UNIT_SET_STORAGE)
							roundType = "ceil"

							valueTitle = string.format("%s [%s]", getText("UC_statistic_siloCapacity"), statUnit.nameI18NShort)
							_, valueSession = g_currentMission.farmSiloTrigger:getAllFillLevels()
						end
					end
				end
			end

			if statUnit ~= ucMod.UNIT_UNKNOWN then
				statistic.valueTitleElem:setText(valueTitle)

				if getIsNumber(valueSession) then
					valueSession = ucMod:convertValue(valueSession, statUnit.index, fillType)
					valueSession = numToStr(valueSession, statUnit.precision, roundType, true, false)
					statistic.valueSessionElem:setText(valueSession)
				end

				if getIsNumber(valueTotal) then
					valueTotal = ucMod:convertValue(valueTotal, statUnit.index, fillType)
					valueTotal = numToStr(valueTotal, statUnit.precision, roundType, true, false)
					statistic.valueTotalElem:setText(valueTotal)
				end
			end
		end
	end
	return unpack(ret)
end

-- [INGAMEMENU FUNCTIONS - ANIMALS PAGE] **************************************************************************************************************************************************************

function UnitConvert:setAnimalAttribute(tbl, attrName, fillTypes, fillLevels, capacity, isPercent)
-- This function adds an entry to the animal statistics screen table
	-- [tbl]:        Table to add the entry to
	-- [attrName]:   Name of the attibute to add (the text you see on screen)
	-- [fillTypes]:  FillType list associated with this statistic
	-- [fillLevels]: Fill levels of the above listed fillTypes
	-- [capacity]:   Capacity of item listed (for showing fill bars)
	-- [isPercent]:  Shows the fill percentage if true (capacity must also be present)

	if not getIsTable(tbl) or not getIsString(attrName) or not fillTypes or not fillLevels
	or (capacity and not getIsNumber(capacity))
	or (isPercent and not getIsBoolean(isPercent))
	then
		raiseError()
		return
	end

	if type(fillTypes) ~= "table" then fillTypes = splitString(" ", fillTypes) end
	if type(fillLevels) ~= "table" then	fillLevels = splitString(" ", fillLevels) end

	local unitSet = ucMod.UNIT_SET_ANIMAL
	local fillUnit = ucMod:getFillUnit(fillTypes, unitSet)

	local fillLevel = 0
	local ucFillLevel = 0
	capacity = Utils.getNoNil(capacity, 0)

	for i = 1, getTableLength(fillTypes) do
		if xAssert(ucMod:getIsFillTypeValid(fillTypes[i])) then
			fillLevels[i] = Utils.getNoNil(fillLevels[i], 0)

			fillLevel = fillLevel + fillLevels[i]
			ucFillLevel = ucFillLevel + ucMod:convertValue(fillLevels[i], fillUnit.index, fillTypes[i])
		end
	end
	ucFillLevel = numToStr(ucFillLevel, fillUnit.precision, "floor", true, false)

	local fillPercent = 0
	if isPercent and capacity > 0 then
		fillPercent = Utils.clamp(fillLevel / capacity, 0, 1)
	else
		isPercent = false
	end

	attrName = string.format("%s [%s]", attrName, fillUnit.nameI18NShort)
	if isPercent then
		ucFillLevel = string.format("%s (%s%%)", ucFillLevel, math.floor(fillPercent * 100))
		table.insert(tbl, {
			name = attrName,
			value = ucFillLevel,
			percent = fillPercent,
			isPercentValue = true})
	else
		table.insert(tbl, {
			name = attrName,
			value = ucFillLevel})
	end
end

-- [INGAMEMENU OVERRIDES - ANIMALS PAGE] **************************************************************************************************************************************************************

function UnitConvert:inGameMenuGetAnimalData(superFunc, ...)
	local ret = {superFunc(self, ...)}

	local animalAttributes = ret[1] -- First return value is the animals attributes table
	if animalAttributes then
		for _, attribute in pairs(animalAttributes) do
			local animalName = attribute.name
			if g_currentMission.husbandries then
				local animalData = g_currentMission.husbandries[animalName]
				if animalData and animalData.hasStatistics then
					attribute.attrSpecific = {}

					if ucMod.animalsByName[animalName].hasPallet then
						local fillType = animalData.palletFillType
						if ucMod:getIsFillTypeValid(fillType) then
							local capacity = math.huge -- Arbitrary number used to show fill level bar even when empty
							local fillLevel = 0

							if animalData.currentPallet then
								capacity = animalData.currentPallet.capacity
								fillLevel = animalData.currentPallet.fillLevel
							end

							local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
							ucMod:setAnimalAttribute(attribute.attrSpecific, attrName, fillType, fillLevel, capacity, true)
						end
					end

					if ucMod.animalsByName[animalName].hasManure then
						local fillType = FillUtil.FILLTYPE_MANURE
						local fillLevel = Utils.getNoNil(animalData.manureFillLevel, 0)
						local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
						ucMod:setAnimalAttribute(attribute.attrSpecific, attrName, fillType, fillLevel)
					end

					if ucMod.animalsByName[animalName].hasLiquidManure and animalData.liquidManureTrigger then
						local fillType = animalData.liquidManureTrigger.fillType
						if ucMod:getIsFillTypeValid(fillType) then
							local fillLevel = Utils.getNoNil(animalData.liquidManureTrigger.fillLevel, 0)
							local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
							ucMod:setAnimalAttribute(attribute.attrSpecific, attrName, fillType, fillLevel)
						end
					end

					if ucMod.animalsByName[animalName].hasMilk then
						local fillType = FillUtil.FILLTYPE_MILK
						local fillLevel = Utils.getNoNil(animalData.fillLevelMilk, 0)
						local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
						ucMod:setAnimalAttribute(attribute.attrSpecific, attrName, fillType, fillLevel)
					end

					if attribute.attrInput then
						local attrInput = {}
						for _, input in ipairs(attribute.attrInput) do
							if string.find(input.name, getText("statistic_cleanliness")) then
								table.insert(attrInput, input)
							end
						end

						if ucMod.animalsByName[animalName].hasWater then
							local fillType = FillUtil.FILLTYPE_WATER
							local capacity = animalData:getCapacity(fillType)
							if capacity and capacity > 0 then
								local fillLevel = Utils.getNoNil(animalData:getFillLevel(fillType), 0)
								local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
								ucMod:setAnimalAttribute(attrInput, attrName, fillType, fillLevel, capacity, true)
							end
						end

						if ucMod.animalsByName[animalName].hasStraw then
							local fillType = FillUtil.FILLTYPE_STRAW
							local capacity = animalData:getCapacity(fillType)
							if capacity and capacity > 0 then
								local fillLevel = Utils.getNoNil(animalData:getFillLevel(fillType), 0)
								local attrName = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
								ucMod:setAnimalAttribute(attrInput, attrName, fillType, fillLevel, capacity, true)
							end
						end

						local animalDesc = AnimalUtil.animals[animalName]
						if animalDesc and FillUtil.foodGroups[animalDesc.index] then
							for _, foodGroup in ipairs(FillUtil.foodGroups[animalDesc.index]) do
								local fillTypes = {}
								local fillLevels = {}
								local capacity = 0

								if foodGroup.fillTypes then
									for fillType, isValid in pairs(foodGroup.fillTypes) do
										if isValid and ucMod:getIsFillTypeValid(fillType) then
											table.insert(fillTypes, fillType)
											table.insert(fillLevels, Utils.getNoNil(animalData:getFillLevel(fillType), 0)) -- TODO: Make sure this is correct
											capacity = math.max(capacity, Utils.getNoNil(animalData:getCapacity(fillType, foodGroup), 0))
										end
									end
								end

								if getTableLength(fillTypes) > 0 then
									local attrName = foodGroup.nameI18N
									ucMod:setAnimalAttribute(attrInput, attrName, fillTypes, fillLevels, capacity, true)
								end
							end
						end
						attribute.attrInput = attrInput
					end
				end
			end
		end
	end
	return unpack(ret)
end

-- [SHOP SCREEN FUNCTIONS] ****************************************************************************************************************************************************************************

function UnitConvert:convertShopScreenAttributes(shopScreen, storeItem)
-- This function converts the various shop screen attributes (working width, capacity, etc)
	-- [shopScreen]: Shop screen element
	-- [storeItem]:  Store item to convert

	if not getIsTable(shopScreen) or not getIsTable(storeItem) then
		raiseError()
		return
	end

	if not shopScreen.attrIcon or not storeItem.specs then
		return
	end

	for idx, attr in pairs(shopScreen.attrIcon) do
		if shopScreen.attrValue
		and shopScreen.attrValue[idx]
		and shopScreen.attrValue[idx].text
		and shopScreen.attrValue[idx].text ~= ""
		then
			-- Fuel Capacity
			if attr.profile == StoreItemsUtil.storeItemSpecsNameToDesc["fuel"].profile then
				local fillType = FillUtil.FILLTYPE_FUEL
				local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_VEHICLE)
				local capacity = Utils.getNoNil(storeItem.specs.fuel, 0)

				capacity = ucMod:convertValue(capacity, fillUnit.index, fillType)
				capacity = numToStr(capacity, fillUnit.precision, "ceil", true, false)

				shopScreen.attrValue[idx]:setText(string.format("%s %s", capacity, fillUnit.nameI18NShort))
			-- Fill Capacity
			elseif attr.profile == StoreItemsUtil.storeItemSpecsNameToDesc["capacity"].profile then
				if storeItem.specs.fillTypes and storeItem.specs.capacity then
					local fillTypes = FillUtil.FILLTYPE_UNKNOWN
					local capacity = Utils.getNoNil(storeItem.specs.capacity.value, 0)

					local categories = storeItem.specs.fillTypes.categoryNames
					if categories then
						fillTypes = ucMod:getFillTypesByCategories(categories)
					end

					if not fillTypes or fillTypes == FillUtil.FILLTYPE_UNKNOWN then
						-- Don"t convert anything with "bale" or "pieces" assigned as a unit
						if storeItem.specs.capacity.unit ~= "$l10n_unit_pieces" and storeItem.specs.capacity.unit ~= "$l10n_unit_bale" then
							fillTypes = storeItem.specs.fillTypes.fillTypes
						end
					end

					if fillTypes and fillTypes ~= FillUtil.FILLTYPE_UNKNOWN then
						local fillType = ucMod:getHeaviestFillType(fillTypes)
						if fillType then
							local fillUnit = ucMod:getFillUnit(fillTypes, ucMod.UNIT_SET_VEHICLE)

							capacity = ucMod:convertValue(capacity, fillUnit.index, fillType)
							capacity = numToStr(capacity, fillUnit.precision, "ceil", true, false)

							shopScreen.attrValue[idx]:setText(string.format("%s %s", capacity, fillUnit.nameI18NShort))
						end
					end
				end
			-- Working Width
			elseif attr.profile == StoreItemsUtil.storeItemSpecsNameToDesc["workingWidth"].profile then
				local lengthUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_LENGTH)

				local workingWidth = Utils.getNoNil(storeItem.specs.workingWidth, 0)
				workingWidth = ucMod:convertValue(workingWidth, lengthUnit.index, false)
				workingWidth = numToStr(workingWidth, lengthUnit.precision, "ceil", true, false)

				shopScreen.attrValue[idx]:setText(string.format("%s %s", workingWidth, lengthUnit.nameI18NShort))
			end
		end
	end
end

-- [SHOP SCREEN OVERRIDES] ****************************************************************************************************************************************************************************

function UnitConvert:shopScreenUpdateAttributes(superFunc, ...)
	local ret = {superFunc(self, ...)}

	if self.currentItemList and getTableLength(self.currentItemList) > 0 and self.currentItemIndex and self.currentItemIndex > 0 then
		ucMod:convertShopScreenAttributes(self, self.currentItemList[self.currentItemIndex])
	end

	return unpack(ret)
end

function UnitConvert:shopConfigScreenUpdateData(superFunc, ...)
	local ret = {superFunc(self, ...)}

	if self.storeItem then
		ucMod:convertShopScreenAttributes(self, self.storeItem)
	end

	return unpack(ret)
end

-- [SILO DIALOG FUNCTIONS] ****************************************************************************************************************************************************************************

function UnitConvert:setSiloDialogTitle(siloDialog)
-- This function simply sets the silo dialog title when opened
	-- [siloDialog]: Silo dialog

	if siloDialog.stationName then
		siloDialog.dialogTitleElement:setText(string.upper(siloDialog.stationName))
	end

	local fillType = siloDialog.selectedFillType
	if ucMod:getIsFillTypeValid(fillType) then
		local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)

		local capacity = Utils.getNoNil(siloDialog.capacity, 0)
		if capacity == math.huge then
			siloDialog.dialogTitleElement2:setText(string.upper(getText("UC_siloDialog_inifiniteCapacity")))
		else
			capacity = ucMod:convertValue(capacity, fillUnit.index, fillType)
			capacity = numToStr(capacity, fillUnit.precision, "ceil", true, false)
			siloDialog.dialogTitleElement2:setText(string.upper(getText("UC_siloDialog_capacity", capacity, fillUnit.nameI18NShort)))
		end
	end
end

function UnitConvert:setSiloDialogText(siloDialog)
	siloDialog.siloText.position[1] = 0
	siloDialog.siloText:setPosition(unpack(siloDialog.siloText.position))
	siloDialog.siloText:setText(siloDialog.fillTypesElement.texts[siloDialog.selectedFillType])
	siloDialog.siloIcon:setVisible(false)
end

-- [SILO DIALOG OVERRIDES] ****************************************************************************************************************************************************************************

function UnitConvert:siloDialogOnClickFillTypes(superFunc, state, ...)
	local ret = {superFunc(self, state, ...)}

	ucMod:setSiloDialogTitle(self)
	ucMod:setSiloDialogText(self)

	return unpack(ret)
end

function UnitConvert:guiShowSiloDialog(superFunc, args, ...)
	local ret = {superFunc(self, args, ...)}

	if getIsTable(args) and getIsTable(args.target) then
		local siloDialog	= self.guis.SiloDialog.target
		local siloTrigger	= args.target

		-- Initialization
		if not siloDialog.unitConvert then siloDialog.unitConvert = {} end
		if not siloDialog.unitConvert.initialized then
			-- Allow special code to run when dialog fill-type is changed
			siloDialog.fillTypesElement.onClickCallback = Utils.overwrittenFunction(siloDialog.fillTypesElement.onClickCallback, ucMod.siloDialogOnClickFillTypes)

			-- Customize fillType text element
			siloDialog.fillTypesElement.textElement:setTextSize(getCorrectTextSize(siloDialog.fillTypesElement.textElement.defaultTextSize) * 0.9)
			--siloDialog.fillTypesElement.textElement.textBold = true

			-- Add an extra title element
			siloDialog.dialogTitleElement2 = siloDialog.dialogTitleElement:clone(siloDialog.dialogTitleElement.parent)

			-- Format title elements
			local dlgTitleText = siloDialog.dialogTitleElement
			local dlgTitleTextPosX, dlgTitleTextPosY = unpack(dlgTitleText.position)
			local dlgTitleTextHeight = dlgTitleText:getTextHeight()

			local dlgTitleText2 = siloDialog.dialogTitleElement2

			dlgTitleText2:setTextSize(dlgTitleText.textSize * 0.65)
			dlgTitleText2:setTextColor(unpack(ucMod.colors.green))

			local dlgTitleText2PosX, dlgTitleText2PosY = unpack(dlgTitleText2.position)
			local dlgTitleText2Height = dlgTitleText2:getTextHeight()

			local dlgTitleBg = dlgTitleText.parent
			local dlgTitleBgWidth, dlgTitleBgHeight = unpack(dlgTitleBg.size)

			dlgTitleTextPosY = dlgTitleBgHeight - dlgTitleTextHeight - 0.006
			dlgTitleText:setPosition(dlgTitleTextPosX, dlgTitleTextPosY)

			dlgTitleText2PosY = 0.001
			dlgTitleText2:setPosition(dlgTitleText2PosX, dlgTitleText2PosY)

			-- Initialization complete
			siloDialog.unitConvert.initialized = true
		end

		-- Add this for future reference
		siloDialog.stationName = siloTrigger.stationName

		if siloDialog.fillTypeMapping then
			for _, fillType in pairs(siloDialog.fillTypeMapping) do
				if ucMod:getIsFillTypeValid(fillType) then
					local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)

					local fillLevel = Utils.getNoNil(siloDialog.fillLevels[fillType], 0)
					fillLevel = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
					fillLevel = numToStr(fillLevel, fillUnit.precision, "ceil", true, false)

					local fillText = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
					siloDialog.fillTypesElement.texts[fillType] = string.format("%s: %s %s", fillText, fillLevel, fillUnit.nameI18NShort)
				end
			end
		end

		ucMod:setSiloDialogTitle(siloDialog)
		ucMod:setSiloDialogText(siloDialog)
	end

	return unpack(ret)
end

-- [FIELDJOBMISSIONSCREEN OVERRIDES] ******************************************************************************************************************************************************************

function UnitConvert:fieldJobMissionScreenSetData(superFunc, npcIndex, fieldNumber, map, area, price, jobReward, jobTitle, jobDesc, vehicles, jobTime, ...)
	local ret = {superFunc(self, npcIndex, fieldNumber, map, area, price, jobReward, jobTitle, jobDesc, vehicles, jobTime, ...)}

	if self.fieldArea and area then
		local ucArea = area
		local areaUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
		ucArea = ucMod:convertValue(ucArea, areaUnit.index, false)
		ucArea = numToStr(ucArea, areaUnit.precision, "ceil", true, false)
		self.fieldArea:setText(string.format("%s %s", ucArea, areaUnit.nameI18NShort))
	end

	return unpack(ret)
end

-- [BGA FUNCTIONS] ************************************************************************************************************************************************************************************

function UnitConvert:getBgaFillTypes(bga)
-- This function retrieves the fillTypes associated with the given bga object
	-- [bga]: Bga object

	local fillTypes = {}

	if bga.tipTriggers then
		for _, tipTrigger in pairs(bga.tipTriggers) do
			for acceptedFillType, isValid in pairs(tipTrigger.acceptedFillTypes) do
				if isValid and ucMod:getIsFillTypeValid(acceptedFillType) then
					if acceptedFillType ~= FillUtil.FILLTYPE_LIQUIDMANURE then -- Liquid manure is handled differently by the BGA
						local added = false
						for _, fillType in pairs(fillTypes) do
							if fillType == acceptedFillType then
								added = true
								break
							end
						end
						if not added then
							table.insert(fillTypes, acceptedFillType)
						end
					end
				end
			end
		end
	end

	if getTableLength(fillTypes) > 0 then return fillTypes end
end

-- [BGA OVERRIDES] ************************************************************************************************************************************************************************************

function UnitConvert:bgaSetFillLevel(superFunc, fillLevel, fillType, fillable, trigger, ...)
	local oldUpdateDisplayNumbers = self.updateDisplayNumbers
	self.updateDisplayNumbers = function(...) end

	local ret = {superFunc(self, fillLevel, fillType, fillable, trigger, ...)}

	self.updateDisplayNumbers = oldUpdateDisplayNumbers

	local bunkerFillLevel = Utils.getNoNil(self.bunkerFillLevel, 0)

	if fillType ~= FillUtil.FILLTYPE_LIQUIDMANURE then
		local fillTypes = ucMod:getBgaFillTypes(self)
		if fillTypes then
			local ucFillType = ucMod:getHeaviestFillType(fillTypes)
			if ucFillType then
				local fillUnit = ucMod:getFillUnit(fillTypes, ucMod.UNIT_SET_STORAGE)
				bunkerFillLevel = ucMod:convertValue(bunkerFillLevel, fillUnit.index, ucFillType)
			end
		end
	end

	self:updateDisplayNumbers(bunkerFillLevel)

	return unpack(ret)
end

-- [BUNKER SILO OVERRIDES] ****************************************************************************************************************************************************************************

function UnitConvert:bunkerSiloUpdate(superFunc, dt, ...)
	local oldAddExtraPrintText = g_currentMission.addExtraPrintText
	g_currentMission.addExtraPrintText = function(...) end

	local ret = {superFunc(self, dt, ...)}

	g_currentMission.addExtraPrintText = oldAddExtraPrintText

   if self:getCanInteract(true) then
        local fillType = self.inputFillType
        if self.state == BunkerSilo.STATE_CLOSED or self.state == BunkerSilo.STATE_FERMENTED or self.state == BunkerSilo.STATE_DRAIN then
            fillType = self.outputFillType
        end

		if ucMod:getIsFillTypeValid(fillType) then
			local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)

			local fillLevel = Utils.getNoNil(self.fillLevel, 0)
			fillLevel = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
			fillLevel = numToStr(fillLevel, fillUnit.precision, "ceil", true, false)

			local fillTypeText = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
			local fillUnitText = fillUnit.nameI18NShort

			if self.state == BunkerSilo.STATE_FILL then
				g_currentMission:addExtraPrintText(string.format("%s: %s [%s]", fillTypeText, fillLevel, fillUnitText))
				g_currentMission:addExtraPrintText(getText("info_compacting")..string.format(" %d%%", self.compactedPercent))
			elseif self.state == BunkerSilo.STATE_CLOSED or self.state == BunkerSilo.STATE_FERMENTED then
				g_currentMission:addExtraPrintText(getText("info_fermenting")..string.format(" (%s): %d%%", fillTypeText, self.fermentingPercent))
			elseif self.state == BunkerSilo.STATE_DRAIN then
				g_currentMission:addExtraPrintText(string.format("%s: %s [%s]", fillTypeText, fillLevel, fillUnitText))
			end
		end
    end

	return unpack(ret)
end

-- [LIQUIDMANUREFILLTRIGGER OVERRIDES] ********************************************************************************************************************************************************************

function UnitConvert:liquidManureFillTriggerUpdate(superFunc, dt, ...)
	local oldAddExtraPrintText = g_currentMission.addExtraPrintText
	g_currentMission.addExtraPrintText = function(...) end

	local ret = {superFunc(self, dt, ...)}

	g_currentMission.addExtraPrintText = oldAddExtraPrintText

    if self:getShowInfo() then
		local fillType = self.fillType
		if ucMod:getIsFillTypeValid(fillType) then
			local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_STORAGE)
			local fillLevel = Utils.getNoNil(self.fillLevel, 0)
			local capacity = Utils.getNoNil(self.capacity, 0)

			fillLevel = Utils.clamp(fillLevel, 0, capacity)

			local fillPercent = 0
			if capacity > 0 then
				fillPercent = fillLevel / capacity
			end

			fillLevel = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
			fillLevel = numToStr(fillLevel, fillUnit.precision, "ceil", true, false)

			local fillTypeText = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
			g_currentMission:addExtraPrintText(string.format("%s [%s]: %s (%s%%)", fillTypeText, fillUnit.nameI18NShort, fillLevel, math.floor(fillPercent * 100)))
		end
    end

	return unpack(ret)
end

-- [WEIGHSTATION OVERRIDES] ***************************************************************************************************************************************************************************

function UnitConvert:weighStationUpdateWeight(superFunc, ...)
	local totalMass = 0
	local fillMassOldTotal = 0
	local fillMassNewTotal = 0

	for vehicle in pairs(self.triggerVehicles) do
		local fillMassOld, fillMassNew = ucMod:getVehicleFillMass(vehicle)

		totalMass = totalMass + vehicle:getTotalMass()
		fillMassOldTotal = fillMassOldTotal + fillMassOld
		fillMassNewTotal = fillMassNewTotal + fillMassNew
	end

	local vehicleMass = totalMass - fillMassOldTotal
	totalMass = vehicleMass + fillMassNewTotal

	local weightUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_WEIGHT)
	totalMass = ucMod:convertValue(totalMass, weightUnit.index, false)

	self:updateDisplayNumbers(totalMass)
end

-- [VEHICLE HUD OVERRIDES] ****************************************************************************************************************************************************************************

function UnitConvert:currentMissionDrawLevelBar(superFunc, levelBar, value, text, textColor, textUnit, textUnitColor, icon, ...)
	local vehicle = self.controlledVehicle

	if Utils.areListsEqual(textUnitColor, ucMod.colors.textUnitColor) then
		textUnitColor = ucMod.colors.textUnitColor2
	end

	local fillType = FillUtil.FILLTYPE_UNKNOWN
	if icon == self.fuelLevelIconOverlay then
		fillType = FillUtil.FILLTYPE_FUEL
	else
		if self.fillTypeOverlays then
			for idx, fillTypeOverlay in pairs(self.fillTypeOverlays) do
				if icon == fillTypeOverlay then
					fillType = idx
					break
				end
			end
		end
	end

	local textFillType = ""
	local textUnit2 = ""

	if ucMod:getIsFillTypeValid(fillType) then
		local fillLevel = strToNum(text)
		if fillLevel then
			local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_VEHICLE)
			fillLevel = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
			fillLevel = numToStr(fillLevel, fillUnit.precision, "ceil", true, false)

			text = fillLevel

			if icon == self.fuelLevelIconOverlay then
				textUnit = fillUnit.nameI18N
				if vehicle and vehicle.unitConvert then
					local fuelUsage = vehicle.unitConvert.fuelUsage
					if fuelUsage and fuelUsage >= 0 and vehicle.isMotorStarted then
						fuelUsage = ucMod:convertValue(fuelUsage, fillUnit.index, fillType)
						fuelUsage = numToStr(fuelUsage, fillUnit.precision, "ceil", true, false)
						textUnit2 = getText("UC_vehicleHud_fuelUsage", fuelUsage, fillUnit.nameI18NShort)
					end
				end
			else
				textFillType = FillUtil.fillTypeIndexToDesc[fillType].nameI18N
				textUnit2 = fillUnit.nameI18N
				if vehicle and vehicle.unitConvert then
					local vehicleYield = vehicle.unitConvert.yield
					if not vehicleYield or vehicle.unitConvert.outputFillType ~= fillType then
						for _, implement in pairs(vehicle.attachedImplements) do
							implement = implement.object
							if implement.unitConvert and implement.unitConvert.yield and implement.unitConvert.outputFillType == fillType then
								vehicleYield = implement.unitConvert.yield
								break
							end
						end
					end
					if vehicleYield and vehicleYield > 0 then
						if ucMod.yieldDisplay ~= ucMod.YIELD_DISPLAY_OFF then
							local unitSet = ucMod.UNIT_SET_VEHICLE
							if ucMod.yieldDisplay == ucMod.YIELD_DISPLAY_PRICE then
								unitSet = ucMod.UNIT_SET_PRICE
							end

							local areaUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
							local yieldUnit = ucMod:getFillUnit(fillType, unitSet)

							vehicleYield = ucMod:convertValue(vehicleYield, yieldUnit.index, fillType)
							vehicleYield = vehicleYield / areaUnit.conversionFactor

							vehicleYield = numToStr(vehicleYield, yieldUnit.precision, "ceil", true, false)
							textUnit2 = getText("UC_vehicleHud_yield", vehicleYield, yieldUnit.nameI18NShort, areaUnit.nameI18NShort)
						end
					end
				end
			end
		end
	end

	local oldIconRender = function(...) end
	if icon and levelBar then
		oldIconRender = icon.render
		icon.render = function(...) end
	end

	local ret = {superFunc(self, levelBar, value, text, textColor, textUnit, textUnitColor, icon, ...)}

	if icon and levelBar then
		icon.render = oldIconRender
		icon:setPosition(icon.x, (levelBar.y - 0.0005))
		icon:render()
	end

	if ucMod:getIsFillTypeValid(fillType) then
		setTextColor(unpack(textUnitColor))
		setTextAlignment(RenderText.ALIGN_RIGHT)

		local textUnit2Size = self.levelTextSmallTextSize * 0.9
		local textUnit2PosX = levelBar.x + levelBar.width
		local textUnit2PosY = levelBar.y - getTextHeight(textUnit2Size, textUnit2) - 0.0005

		renderText(textUnit2PosX, textUnit2PosY, textUnit2Size, textUnit2)

		setTextColor(unpack(ucMod.colors.white))
		setTextAlignment(RenderText.ALIGN_LEFT)

		local textFillTypeSize = textUnit2Size
		local textFillTypePosX = icon.x
		local textFillTypePosY = textUnit2PosY

		renderText(textFillTypePosX, textFillTypePosY, textFillTypeSize, textFillType)

		setTextColor(1, 1, 1, 1)
	end

	return unpack(ret)
end

function UnitConvert:vehicleHudUtilsSetHudValue(superFunc, huds, value, maxValue, ...)
	local doUpdate = false
	for _, hud in pairs(huds) do
		if hud.numbers then
			doUpdate = true
			break
		end
	end

	if doUpdate then
		local fillType = FillUtil.FILLTYPE_UNKNOWN
		if (self.fuelFillLevelHud and huds == self.fuelFillLevelHud) or (self.fuelUsageHud and huds == self.fuelUsageHud) then
			fillType = FillUtil.FILLTYPE_FUEL
		elseif self.workedHectaresHud and huds == self.workedHectaresHud then
			local areaUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
			workedHectares = Utils.getNoNil(value, 0)
			value = ucMod:convertValue(workedHectares, areaUnit.index, false)
		else
			if self.fillUnits then
				for _, fillUnit in pairs(self.fillUnits) do
					if fillUnit.fillLevelHud and huds == fillUnit.fillLevelHud then
						fillType = fillUnit.lastValidFillType
						break
					end
				end
			end
		end

		if ucMod:getIsFillTypeValid(fillType) then
			local fillLevel = Utils.getNoNil(value, 0)
			local capacity = Utils.getNoNil(maxValue, fillLevel)

			local fillUnit = ucMod:getFillUnit(fillType, ucMod.UNIT_SET_VEHICLE)

			value = ucMod:convertValue(fillLevel, fillUnit.index, fillType)
			maxValue = ucMod:convertValue(capacity, fillUnit.index, fillType)

			value = Utils.clamp(value, 0, maxValue)
		end
	end

	local ret = {superFunc(self, huds, value, maxValue, ...)}
	return unpack(ret)
end

-- [VEHICLE OVERRIDES] ********************************************************************************************************************************************************************************
-- The vehicle stuff really should be a specialization, and maybe will be in FS19. For now it was much easier to do things this way since I'm really only grabbing a couple values
-- here and there, and I'm hooking the code of ALL vehicles (including trains) instead of adding new features.

function UnitConvert:vehicleAddCutterArea(superFunc, cutter, area, realArea, inputFruitType, outputFillType, ...)
	local ret = {superFunc(self, cutter, area, realArea, inputFruitType, outputFillType, ...)}

	local yield = 0

	if self.unitConvert.updateYield then
		if ucMod:getIsFillTypeValid(outputFillType) then
			local literPerSqm = 1
			if ucMod:getIsFruitTypeValid(inputFruitType) then
				literPerSqm = FruitUtil.fruitIndexToDesc[inputFruitType].literPerSqm
			end

			local yieldMultiplier = 0
			if realArea and realArea > 0 then
				yieldMultiplier = (area / realArea) * self.threshingScale
			end

			yield = literPerSqm * yieldMultiplier
			yield = yield * 10000 --Convert to hectares
		end
	end

	self.unitConvert.yield = yield
	self.unitConvert.inputFruitType = inputFruitType
	self.unitConvert.outputFillType = outputFillType

	return unpack(ret)
end

function UnitConvert:vehicleUpdate(superFunc, dt, ...)
	local ret = {superFunc(self, dt, ...)}

	-- Initialization
	if self.unitConvert and not self.unitConvert.isInitialized then
		if self.fuelFillLevel and Utils.getNoNil(self.fuelCapacity, 0) > 0 then
			self.unitConvert.fuelUsage = 0
		end

		if SpecializationUtil.hasSpecialization(Combine, self.specializations) and self.addCutterArea then
			self.addCutterArea = Utils.overwrittenFunction(self.addCutterArea, ucMod.vehicleAddCutterArea)
			self.unitConvert.yield = 0
		end

		self.unitConvert.isInitialized = true
	end

	if not self.attacherVehicle then
		if self:getIsActive() and self:getIsActiveForInput(false, false) then
			local showYieldToggle = false
			if self.unitConvert and self.unitConvert.yield then
				showYieldToggle = true
			else
				for _, implement in pairs(self.attachedImplements) do
					 implement = implement.object
					 if implement and implement.unitConvert and implement.unitConvert.yield then
						showYieldToggle = true
						break
					end
				end
			end
			if showYieldToggle then
				g_currentMission:addHelpButtonText(getText("UC_help_toggleYield"), InputBinding.UC_TOGGLE_YIELD, nil, GS_PRIO_VERY_LOW)
				g_currentMission:addExtraPrintText(getText("UC_help_yieldDisplay", ucMod.yieldDisplayTypes[ucMod.yieldDisplay]))
				if InputBinding.hasEvent(InputBinding.UC_TOGGLE_YIELD) then
					local currentValue = ucMod.yieldDisplay
					currentValue = currentValue + 1
					if currentValue > getTableLength(ucMod.yieldDisplayTypes) then currentValue = 1 end

					ucMod.yieldDisplay = currentValue
					ucMod:saveConfig()
				end
			end
		end
	end

	return unpack(ret)
end
Vehicle.update = Utils.overwrittenFunction(Vehicle.update, ucMod.vehicleUpdate)

function UnitConvert:vehicleUpdateTick(superFunc, dt, ...)
	local ret = {superFunc(self, dt, ...)}

	if self.isServer then
		if self.unitConvert then
			self.unitConvert.updateTickDT = Utils.getNoNil(self.unitConvert.updateTickDT, 0) + dt
			if self.unitConvert.updateTickDT > 1000 then
				-- Fuel Usage
				if self.unitConvert.fuelUsage then
					self.unitConvert.lastFuelFillLevel = Utils.getNoNil(self.unitConvert.lastFuelFillLevel, self.fuelFillLevel)

					local fuelUsage = math.max(0, (self.unitConvert.lastFuelFillLevel - self.fuelFillLevel))
					self.unitConvert.fuelUsage = (fuelUsage / self.unitConvert.updateTickDT) * 1000 * 60 * 60
					self.unitConvert.lastFuelFillLevel = self.fuelFillLevel
				end

				-- Yield
				if self.unitConvert.yield and self.lastAreaZeroTime then
					self.unitConvert.updateYield = true
					if self.lastAreaZeroTime > 1000 then
						self.unitConvert.updateYield = false
						self.unitConvert.yield = 0
					end
				end

				self.unitConvert.updateTickDT = 0
			end
		end
	end

	return unpack(ret)
end
Vehicle.updateTick = Utils.overwrittenFunction(Vehicle.updateTick, ucMod.vehicleUpdateTick)

function UnitConvert:vehicleReadStream(superFunc, streamId, connection, ...)
	local ret = {superFunc(self, streamId, connection, ...)}
	return unpack(ret)
end
--Vehicle.readStream = Utils.overwrittenFunction(Vehicle.readStream, ucMod.vehicleReadStream)

function UnitConvert:vehicleWriteStream(superFunc, streamId, connection, ...)
	local ret = {superFunc(self, streamId, connection, ...)}
	return unpack(ret)
end
--Vehicle.writeStream = Utils.overwrittenFunction(Vehicle.writeStream, ucMod.vehicleWriteStream)

function UnitConvert:vehicleReadUpdateStream(superFunc, streamId, timeStamp, connection, ...)
	local ret = {superFunc(self, streamId, timeStamp, connection, ...)}

	if connection.isServer then
		local fuelUsagePct		= streamReadUIntN(streamId, 14)
		local yieldPct			= streamReadUInt16(streamId)
		local inputFruitType	= streamReadUInt8(streamId)
		local outputFillType	= streamReadUInt8(streamId)

		if self.unitConvert then
			if self.unitConvert.fuelUsage then
				fuelUsagePct = fuelUsagePct / 16383

				local fuelUsage = Utils.clamp((self.fuelCapacity * fuelUsagePct), 0, self.fuelCapacity)
				self.unitConvert.fuelUsage = fuelUsage
			end

			if self.unitConvert.yield then
				local yield = 0
				if yieldPct > 0 and ucMod:getIsFruitTypeValid(inputFruitType) then
					local fruitType = FruitUtil.fruitIndexToDesc[inputFruitType]
					yieldPct = yieldPct / 10000 -- Convert percentage to decimal (10000 (100.00%) / 10000 = 1)
					yield = fruitType.literPerSqm * yieldPct * 10000  -- Convert to hectares (x 10000)
				end
				self.unitConvert.yield = yield
				self.unitConvert.inputFruitType = inputFruitType
				self.unitConvert.outputFillType = outputFillType
			end
		end
	end

	return unpack(ret)
end
Vehicle.readUpdateStream = Utils.overwrittenFunction(Vehicle.readUpdateStream, ucMod.vehicleReadUpdateStream)

function UnitConvert:vehicleWriteUpdateStream(superFunc, streamId, connection, dirtyMask, ...)
	local ret = {superFunc(self, streamId, connection, dirtyMask, ...)}

	if not connection.isServer then
		local fuelUsagePct = 0
		local yieldPct = 0
		local inputFruitType = FruitUtil.FRUITTYPE_UNKNOWN
		local outputFillType = FillUtil.FILLTYPE_UNKNOWN

		if self.unitConvert then
			if self.unitConvert.fuelUsage then
				fuelUsagePct = Utils.clamp((self.unitConvert.fuelUsage / self.fuelCapacity), 0, 1)
				fuelUsagePct = math.floor(fuelUsagePct * 16383)
			end

			if self.unitConvert.yield then
				inputFruitType = Utils.getNoNil(self.unitConvert.inputFruitType, inputFruitType)
				outputFillType = Utils.getNoNil(self.unitConvert.outputFillType, outputFillType)

				if self.unitConvert.yield > 0 and ucMod:getIsFruitTypeValid(inputFruitType) then
					local fruitType = FruitUtil.fruitIndexToDesc[inputFruitType]
					yieldPct = (self.unitConvert.yield / 10000) / fruitType.literPerSqm
					yieldPct = math.floor(Utils.clamp(yieldPct, 0, 6) * 10000) -- Up to 6x normal yield (6 = 600% * 10000 = 60000 < 65535 = 16 bit)
				end
			end
		end

		streamWriteUIntN(streamId, fuelUsagePct, 14)
		streamWriteUInt16(streamId, yieldPct)
		streamWriteUInt8(streamId, inputFruitType)
		streamWriteUInt8(streamId, outputFillType)
	end

	return unpack(ret)
end
Vehicle.writeUpdateStream = Utils.overwrittenFunction(Vehicle.writeUpdateStream, ucMod.vehicleWriteUpdateStream)

function UnitConvert:vehicleLoad(superFunc, ...)
	local ret = {superFunc(self, ...)}

	if not self.unitConvert then self.unitConvert = {} end

	return unpack(ret)
end
Vehicle.load = Utils.overwrittenFunction(Vehicle.load, ucMod.vehicleLoad)

-- [BUG FIXES] ****************************************************************************************************************************************************************************************

-- #001: Game still shows "start filling" text if trailer is only slightly filled and silo is empty.
	-- This may be something they implemented with the "fillTypeChangeThreshold" variable for fillable vehicles.
	-- If so, I couldn"t find a way to actually override a fillType at a silo in-game.

function UnitConvert:siloTriggerGetIsActivatable(superFunc, ...)
	if self.isEnabled then
		if self.isFilling then
			if self.siloTrailer:getRootAttacherVehicle() == g_currentMission.controlledVehicle then
				return true
			end
		else
			if self.siloTrailer and self.activeTriggers >= 4 then
				local trailer = self.siloTrailer
				if trailer:getRootAttacherVehicle() == g_currentMission.controlledVehicle and trailer:getAllowFillFromAir() then
					if trailer:getFillLevel() == 0 then
						return true
					else
						local fillTypes = trailer:getCurrentFillTypes()
						for _,fillType in pairs(fillTypes) do
							if self.fillTypes[fillType] and trailer:getFillLevel(fillType) < trailer:getCapacity() and self:getFillLevel(fillType) > 0 then
								return true
							end
						end
					end
				end
			end
		end
	end
	return false
end
if isDev then
	SiloTrigger.getIsActivatable = Utils.overwrittenFunction(SiloTrigger.getIsActivatable, ucMod.siloTriggerGetIsActivatable)
end

addModEventListener(UnitConvert)
